# Copyright 2014 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from oslo_log import log
from tempest.common import utils
from tempest.common.utils.linux import remote_client
from tempest.common.utils import net_downtime
from tempest.common import waiters
from tempest import config
from tempest.lib.common import api_version_request
from tempest.lib import decorators
from tempest.scenario import manager

CONF = config.CONF

LOG = log.getLogger(__name__)


class BaseTestNetworkAdvancedServerOps(manager.NetworkScenarioTest):
    """Base class for defining methods used in tests."""

    @classmethod
    def skip_checks(cls):
        super(BaseTestNetworkAdvancedServerOps, cls).skip_checks()
        if not (CONF.network.project_networks_reachable or
                CONF.network.public_network_id):
            msg = ('Either project_networks_reachable must be "true", or '
                   'public_network_id must be defined.')
            raise cls.skipException(msg)
        if not CONF.network_feature_enabled.floating_ips:
            raise cls.skipException("Floating ips are not available")

    @classmethod
    def setup_clients(cls):
        super(BaseTestNetworkAdvancedServerOps, cls).setup_clients()
        cls.admin_servers_client = cls.os_admin.servers_client
        cls.sec_group_rules_client = \
            cls.os_primary.security_group_rules_client
        cls.sec_groups_client = cls.os_primary.security_groups_client
        cls.keypairs_client = cls.os_primary.keypairs_client
        cls.floating_ips_client = cls.os_primary.floating_ips_client
        cls.servers_client = cls.os_primary.servers_client

    @classmethod
    def setup_credentials(cls):
        # Create no network resources for these tests.
        cls.set_network_resources()
        super(BaseTestNetworkAdvancedServerOps, cls).setup_credentials()

    def _setup_server(self, keypair, host_spec=None):
        security_groups = []
        if utils.is_extension_enabled('security-group', 'network'):
            sec_args = {
                'security_group_rules_client':
                self.sec_group_rules_client,
                'security_groups_client':
                self.sec_groups_client
            }
            security_group = self.create_security_group(**sec_args)
            security_groups = [{'name': security_group['name']}]
        network, _, _ = self.setup_network_subnet_with_router()
        server_args = {
            'networks': [{'uuid': network['id']}],
            'key_name': keypair['name'],
            'security_groups': security_groups,
        }

        if host_spec is not None:
            server_args['host'] = host_spec
            # by default, host can be specified by administrators only
            server_args['clients'] = self.os_admin

        server = self.create_server(**server_args)
        return server

    def _setup_network(self, server, keypair):
        public_network_id = CONF.network.public_network_id
        floating_ip = self.create_floating_ip(
            server, public_network_id, client=self.floating_ips_client)
        # Verify that we can indeed connect to the server before we mess with
        # it's state
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

        return floating_ip

    def _check_network_connectivity(self, server, keypair, floating_ip,
                                    should_connect=True,
                                    username=CONF.validation.image_ssh_user):
        private_key = keypair['private_key']
        self.check_tenant_network_connectivity(
            server, username, private_key,
            should_connect=should_connect,
            servers_for_debug=[server])
        floating_ip_addr = floating_ip['floating_ip_address']
        # Check FloatingIP status before checking the connectivity
        self.check_floating_ip_status(floating_ip, 'ACTIVE')
        self.check_vm_connectivity(floating_ip_addr, username,
                                   private_key, should_connect,
                                   'Public network connectivity check failed',
                                   server)

    def _wait_server_status_and_check_network_connectivity(
        self, server, keypair, floating_ip,
        username=CONF.validation.image_ssh_user):
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'ACTIVE')
        self._check_network_connectivity(server, keypair, floating_ip,
                                         username=username)

    def _test_server_connectivity_resize(self, src_host=None):
        resize_flavor = CONF.compute.flavor_ref_alt
        keypair = self.create_keypair()
        server = self._setup_server(keypair, src_host)
        if src_host:
            server_host = self.get_host_for_server(server['id'])
            self.assertEqual(server_host, src_host)
        floating_ip = self._setup_network(server, keypair)
        self.servers_client.resize_server(server['id'],
                                          flavor_ref=resize_flavor)
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'VERIFY_RESIZE')
        self.servers_client.confirm_resize_server(server['id'])
        server = self.servers_client.show_server(server['id'])['server']
        # Nova API > 2.46 no longer includes flavor.id, and schema check
        # will cover whether 'id' should be in flavor
        if server['flavor'].get('id'):
            self.assertEqual(resize_flavor, server['flavor']['id'])
        else:
            flavor = self.flavors_client.show_flavor(resize_flavor)['flavor']
            self.assertEqual(flavor['name'], server['flavor']['original_name'])
            for key in ['ram', 'vcpus', 'disk']:
                self.assertEqual(flavor[key], server['flavor'][key])
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

    def _test_server_connectivity_cold_migration(self, source_host=None,
                                                 dest_host=None):
        keypair = self.create_keypair(client=self.keypairs_client)
        server = self._setup_server(keypair, source_host)
        floating_ip = self._setup_network(server, keypair)
        src_host = self.get_host_for_server(server['id'])
        if source_host:
            self.assertEqual(src_host, source_host)
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

        self.admin_servers_client.migrate_server(
            server['id'], host=dest_host)
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'VERIFY_RESIZE')
        self.servers_client.confirm_resize_server(server['id'])
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)
        dst_host = self.get_host_for_server(server['id'])
        if dest_host:
            self.assertEqual(dst_host, dest_host)
        self.assertNotEqual(src_host, dst_host)

    def _test_server_connectivity_live_migration(self, source_host=None,
                                                 dest_host=None,
                                                 migration=False):
        keypair = self.create_keypair(client=self.keypairs_client)
        server = self._setup_server(keypair, source_host)
        floating_ip = self._setup_network(server, keypair)
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

        block_migration = (CONF.compute_feature_enabled.
                           block_migration_for_live_migration)
        src_host = self.get_host_for_server(server['id'])
        if source_host:
            self.assertEqual(src_host, source_host)

        downtime_meter = net_downtime.NetDowntimeMeter(
            floating_ip['floating_ip_address'])
        self.useFixture(downtime_meter)

        metadata_downtime_meter = net_downtime.MetadataDowntimeMeter(
            remote_client.RemoteClient(floating_ip['floating_ip_address'],
                                       CONF.validation.image_ssh_user,
                                       pkey=keypair['private_key']))
        self.useFixture(metadata_downtime_meter)

        migration_kwargs = {'host': None, 'block_migration': block_migration}

        # check if microversion is less than 2.25 because of
        # disk_over_commit is depracted since compute api version 2.25
        # if min_microversion is None, it runs on version < 2.25
        min_v = api_version_request.APIVersionRequest(
            CONF.compute.min_microversion)
        api_v = api_version_request.APIVersionRequest('2.25')
        if not migration and (CONF.compute.min_microversion is None or
                              min_v < api_v):
            migration_kwargs['disk_over_commit'] = False

        if dest_host:
            migration_kwargs['host'] = dest_host

        self.admin_servers_client.live_migrate_server(
            server['id'], **migration_kwargs)
        waiters.wait_for_server_status(self.servers_client,
                                       server['id'], 'ACTIVE')

        dst_host = self.get_host_for_server(server['id'])
        if dest_host:
            self.assertEqual(dst_host, dest_host)

        self.assertNotEqual(src_host, dst_host, 'Server did not migrate')

        # we first wait until the VM replies pings again, then check the
        # network downtime
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

        downtime = downtime_meter.get_downtime()
        self.assertIsNotNone(downtime)
        LOG.debug("Downtime seconds measured with downtime_meter = %r",
                  downtime)
        allowed_downtime = CONF.validation.allowed_network_downtime
        self.assertLessEqual(
            downtime, allowed_downtime,
            "Downtime of {} seconds is higher than expected '{}'".format(
                downtime, allowed_downtime))

        metadata_downtime_results = metadata_downtime_meter.get_results()
        self.assertGreater(metadata_downtime_results['successes'], 0)
        LOG.debug("Metadata Downtime seconds measured = %r",
                  metadata_downtime_results['downtime'])
        allowed_metadata_downtime = CONF.validation.allowed_metadata_downtime
        metadata_downtime_failed = \
            metadata_downtime_results['downtime']['FAILED']
        self.assertLessEqual(
            metadata_downtime_failed, allowed_metadata_downtime,
            "Metadata downtime: {} seconds is higher than expected: {}".format(
                metadata_downtime_failed, allowed_metadata_downtime))

    def _test_server_connectivity_cold_migration_revert(self, source_host=None,
                                                        dest_host=None):
        keypair = self.create_keypair(client=self.keypairs_client)
        server = self._setup_server(keypair, source_host)
        floating_ip = self._setup_network(server, keypair)
        src_host = self.get_host_for_server(server['id'])
        if source_host:
            self.assertEqual(src_host, source_host)
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

        self.admin_servers_client.migrate_server(
            server['id'], host=dest_host)
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'VERIFY_RESIZE')
        if dest_host:
            self.assertEqual(dest_host,
                             self.get_host_for_server(server['id']))
        self.servers_client.revert_resize_server(server['id'])
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)
        dst_host = self.get_host_for_server(server['id'])

        self.assertEqual(src_host, dst_host)


class TestNetworkAdvancedServerOps(BaseTestNetworkAdvancedServerOps):
    """Check VM connectivity after some advanced instance operations executed:

     * Stop/Start an instance
     * Reboot an instance
     * Rebuild an instance
     * Pause/Unpause an instance
     * Suspend/Resume an instance
    """

    @decorators.idempotent_id('61f1aa9a-1573-410e-9054-afa557cab021')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_server_connectivity_stop_start(self):
        keypair = self.create_keypair()
        server = self._setup_server(keypair)
        floating_ip = self._setup_network(server, keypair)
        self.servers_client.stop_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'SHUTOFF')
        self._check_network_connectivity(server, keypair, floating_ip,
                                         should_connect=False)
        self.servers_client.start_server(server['id'])
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

    @decorators.idempotent_id('7b6860c2-afa3-4846-9522-adeb38dfbe08')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_server_connectivity_reboot(self):
        keypair = self.create_keypair()
        server = self._setup_server(keypair)
        floating_ip = self._setup_network(server, keypair)
        self.servers_client.reboot_server(server['id'], type='SOFT')
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

    @decorators.idempotent_id('88a529c2-1daa-4c85-9aec-d541ba3eb699')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_server_connectivity_rebuild(self):
        keypair = self.create_keypair()
        server = self._setup_server(keypair)
        floating_ip = self._setup_network(server, keypair)
        image_ref_alt = CONF.compute.image_ref_alt
        username_alt = CONF.validation.image_alt_ssh_user
        self.servers_client.rebuild_server(server['id'],
                                           image_ref=image_ref_alt)
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip, username_alt)

    @decorators.idempotent_id('2b2642db-6568-4b35-b812-eceed3fa20ce')
    @testtools.skipUnless(CONF.compute_feature_enabled.pause,
                          'Pause is not available.')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_server_connectivity_pause_unpause(self):
        keypair = self.create_keypair()
        server = self._setup_server(keypair)
        floating_ip = self._setup_network(server, keypair)
        self.servers_client.pause_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'PAUSED')
        self._check_network_connectivity(server, keypair, floating_ip,
                                         should_connect=False)
        self.servers_client.unpause_server(server['id'])
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

    @decorators.idempotent_id('5cdf9499-541d-4923-804e-b9a60620a7f0')
    @testtools.skipUnless(CONF.compute_feature_enabled.suspend,
                          'Suspend is not available.')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_server_connectivity_suspend_resume(self):
        keypair = self.create_keypair()
        server = self._setup_server(keypair)
        floating_ip = self._setup_network(server, keypair)
        self.servers_client.suspend_server(server['id'])
        waiters.wait_for_server_status(self.servers_client, server['id'],
                                       'SUSPENDED')
        self._check_network_connectivity(server, keypair, floating_ip,
                                         should_connect=False)
        self.servers_client.resume_server(server['id'])
        self._wait_server_status_and_check_network_connectivity(
            server, keypair, floating_ip)

    @decorators.idempotent_id('719eb59d-2f42-4b66-b8b1-bb1254473967')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize is not available.')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_server_connectivity_resize(self):
        self._test_server_connectivity_resize()

    @decorators.idempotent_id('a4858f6c-401e-4155-9a49-d5cd053d1a2f')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration is not available.')
    @testtools.skipUnless(CONF.compute.min_compute_nodes > 1,
                          'Less than 2 compute nodes, skipping multinode '
                          'tests.')
    @decorators.attr(type=['slow', 'multinode'])
    @utils.services('compute', 'network')
    def test_server_connectivity_cold_migration(self):
        self._test_server_connectivity_cold_migration()

    @decorators.idempotent_id('03fd1562-faad-11e7-9ea0-fa163e65f5ce')
    @testtools.skipUnless(CONF.compute_feature_enabled.live_migration,
                          'Live migration is not available.')
    @testtools.skipUnless(CONF.compute.min_compute_nodes > 1,
                          'Less than 2 compute nodes, skipping multinode '
                          'tests.')
    @decorators.attr(type=['slow', 'multinode'])
    @utils.services('compute', 'network')
    def test_server_connectivity_live_migration(self):
        self._test_server_connectivity_live_migration()

    @decorators.idempotent_id('25b188d7-0183-4b1e-a11d-15840c8e2fd6')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration is not available.')
    @testtools.skipUnless(CONF.compute.min_compute_nodes > 1,
                          'Less than 2 compute nodes, skipping multinode '
                          'tests.')
    @decorators.attr(type=['slow', 'multinode'])
    @utils.services('compute', 'network')
    def test_server_connectivity_cold_migration_revert(self):
        self._test_server_connectivity_cold_migration_revert()


class TestNetworkAdvancedServerMigrationWithHost(
    BaseTestNetworkAdvancedServerOps):

    """Check VM connectivity with specifying source and destination hosts:

    * Resize an instance by creating server on configured source host
    * Migrate server by creating it on configured source host and migrate it
        - Cold Migration
        - Cold Migration with revert
        - Live Migration
    """
    credentials = ['primary', 'admin']
    compute_min_microversion = "2.74"

    @classmethod
    def skip_checks(cls):
        super(TestNetworkAdvancedServerMigrationWithHost, cls).skip_checks()
        if not (CONF.compute.migration_source_host or
                CONF.compute.migration_dest_host):
            raise cls.skipException("migration_source_host or "
                                    "migration_dest_host is required")
        if (CONF.compute.migration_source_host and
            CONF.compute.migration_dest_host and
            CONF.compute.migration_source_host ==
            CONF.compute.migration_dest_host):
            raise cls.skipException("migration_source_host and "
                                    "migration_dest_host must be different")

    @classmethod
    def setup_clients(cls):
        super(BaseTestNetworkAdvancedServerOps, cls).setup_clients()
        cls.sec_group_rules_client = \
            cls.os_admin.security_group_rules_client
        cls.sec_groups_client = cls.os_admin.security_groups_client
        cls.keypairs_client = cls.os_admin.keypairs_client
        cls.floating_ips_client = cls.os_admin.floating_ips_client
        cls.servers_client = cls.os_admin.servers_client
        cls.admin_servers_client = cls.os_admin.servers_client

    @decorators.idempotent_id('06e23934-79ae-11ee-b962-0242ac120002')
    @testtools.skipUnless(CONF.compute_feature_enabled.resize,
                          'Resize is not available.')
    @decorators.attr(type='slow')
    @utils.services('compute', 'network')
    def test_server_connectivity_resize(self):
        source_host = CONF.compute.migration_source_host
        self._test_server_connectivity_resize(src_host=source_host)

    @decorators.idempotent_id('14f0c9e6-79ae-11ee-b962-0242ac120002')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration is not available.')
    @testtools.skipUnless(CONF.compute.min_compute_nodes > 1,
                          'Less than 2 compute nodes, skipping multinode '
                          'tests.')
    @decorators.attr(type=['slow', 'multinode'])
    @utils.services('compute', 'network')
    def test_server_connectivity_cold_migration(self):
        source_host = CONF.compute.migration_source_host
        dest_host = CONF.compute.migration_dest_host
        self._test_server_connectivity_cold_migration(
            source_host=source_host, dest_host=dest_host)

    @decorators.idempotent_id('1c13933e-79ae-11ee-b962-0242ac120002')
    @testtools.skipUnless(CONF.compute_feature_enabled.live_migration,
                          'Live migration is not available.')
    @testtools.skipUnless(CONF.compute.min_compute_nodes > 1,
                          'Less than 2 compute nodes, skipping multinode '
                          'tests.')
    @decorators.attr(type=['slow', 'multinode'])
    @utils.services('compute', 'network')
    def test_server_connectivity_live_migration(self):
        source_host = CONF.compute.migration_source_host
        dest_host = CONF.compute.migration_dest_host
        self._test_server_connectivity_live_migration(
            source_host=source_host, dest_host=dest_host, migration=True)

    @decorators.idempotent_id('2627789a-79ae-11ee-b962-0242ac120002')
    @testtools.skipUnless(CONF.compute_feature_enabled.cold_migration,
                          'Cold migration is not available.')
    @testtools.skipUnless(CONF.compute.min_compute_nodes > 1,
                          'Less than 2 compute nodes, skipping multinode '
                          'tests.')
    @decorators.attr(type=['slow', 'multinode'])
    @utils.services('compute', 'network')
    def test_server_connectivity_cold_migration_revert(self):
        source_host = CONF.compute.migration_source_host
        dest_host = CONF.compute.migration_dest_host
        self._test_server_connectivity_cold_migration_revert(
            source_host=source_host, dest_host=dest_host)



# Copyright 2014 NEC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils
from tempest.tests import base


class TestDataUtils(base.TestCase):

    def test_rand_uuid(self):
        actual = data_utils.rand_uuid()
        self.assertIsInstance(actual, str)
        self.assertRegex(actual, "^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]"
                         "{4}-[0-9a-f]{4}-[0-9a-f]{12}$")
        actual2 = data_utils.rand_uuid()
        self.assertNotEqual(actual, actual2)

    def test_rand_uuid_hex(self):
        actual = data_utils.rand_uuid_hex()
        self.assertIsInstance(actual, str)
        self.assertRegex(actual, "^[0-9a-f]{32}$")

        actual2 = data_utils.rand_uuid_hex()
        self.assertNotEqual(actual, actual2)

    def test_rand_name_with_default_prefix(self):
        actual = data_utils.rand_name('foo')
        self.assertIsInstance(actual, str)
        self.assertTrue(actual.startswith('tempest-foo'))
        actual2 = data_utils.rand_name('foo')
        self.assertTrue(actual2.startswith('tempest-foo'))
        self.assertNotEqual(actual, actual2)

    def test_rand_name_with_none_prefix(self):
        actual = data_utils.rand_name('foo', prefix=None)
        self.assertIsInstance(actual, str)
        self.assertTrue(actual.startswith('foo'))
        actual2 = data_utils.rand_name('foo', prefix=None)
        self.assertTrue(actual2.startswith('foo'))
        self.assertNotEqual(actual, actual2)

    def test_rand_name_with_prefix(self):
        actual = data_utils.rand_name(prefix='prefix-str')
        self.assertIsInstance(actual, str)
        self.assertRegex(actual, "^prefix-str-")
        actual2 = data_utils.rand_name(prefix='prefix-str')
        self.assertNotEqual(actual, actual2)

    def test_rand_password(self):
        actual = data_utils.rand_password()
        self.assertIsInstance(actual, str)
        self.assertRegex(actual, "[A-Za-z0-9~!@#%^&*_=+]{15,}")
        actual2 = data_utils.rand_password()
        self.assertNotEqual(actual, actual2)

    def test_rand_password_with_len(self):
        actual = data_utils.rand_password(8)
        self.assertIsInstance(actual, str)
        self.assertEqual(len(actual), 8)
        self.assertRegex(actual, "[A-Za-z0-9~!@#%^&*_=+]{8}")
        actual2 = data_utils.rand_password(8)
        self.assertNotEqual(actual, actual2)

    def test_rand_password_with_len_2(self):
        actual = data_utils.rand_password(2)
        self.assertIsInstance(actual, str)
        self.assertEqual(len(actual), 3)
        self.assertRegex(actual, "[A-Za-z0-9~!@#%^&*_=+]{3}")
        actual2 = data_utils.rand_password(2)
        # NOTE(masayukig): Originally, we checked that the actual and actual2
        # are different each other. But only 3 letters can be the same value
        # in a very rare case. So, we just check the length here, too,
        # just in case.
        self.assertEqual(len(actual2), 3)

    def test_rand_url(self):
        actual = data_utils.rand_url()
        self.assertIsInstance(actual, str)
        self.assertRegex(actual, r"^https://url-[0-9]*\.com$")
        actual2 = data_utils.rand_url()
        self.assertNotEqual(actual, actual2)

    def test_rand_int(self):
        actual = data_utils.rand_int_id()
        self.assertIsInstance(actual, int)

        actual2 = data_utils.rand_int_id()
        self.assertNotEqual(actual, actual2)

    def test_rand_infiniband_guid_address(self):
        actual = data_utils.rand_infiniband_guid_address()
        self.assertIsInstance(actual, str)
        self.assertRegex(actual, "^([0-9a-f][0-9a-f]:){7}"
                         "[0-9a-f][0-9a-f]$")

        actual2 = data_utils.rand_infiniband_guid_address()
        self.assertNotEqual(actual, actual2)

    def test_rand_mac_address(self):
        actual = data_utils.rand_mac_address()
        self.assertIsInstance(actual, str)
        self.assertRegex(actual, "^([0-9a-f][0-9a-f]:){5}"
                         "[0-9a-f][0-9a-f]$")

        actual2 = data_utils.rand_mac_address()
        self.assertNotEqual(actual, actual2)

    def test_parse_image_id(self):
        actual = data_utils.parse_image_id("/foo/bar/deadbeaf")
        self.assertEqual("deadbeaf", actual)

    def test_arbitrary_string(self):
        actual = data_utils.arbitrary_string()
        self.assertEqual(actual, "test")
        actual = data_utils.arbitrary_string(size=30, base_text="abc")
        self.assertEqual(actual, "abc" * int(30 / len("abc")))
        actual = data_utils.arbitrary_string(size=5, base_text="deadbeaf")
        self.assertEqual(actual, "deadb")

    def test_random_bytes(self):
        actual = data_utils.random_bytes()  # default size=1024
        self.assertIsInstance(actual, bytes)
        self.assertEqual(1024, len(actual))
        actual2 = data_utils.random_bytes()
        self.assertNotEqual(actual, actual2)

        actual = data_utils.random_bytes(size=2048)
        self.assertEqual(2048, len(actual))

    def test_chunkify(self):
        data = "aaa"
        chunks = data_utils.chunkify(data, 2)
        self.assertEqual("aa", next(chunks))
        self.assertEqual("a", next(chunks))
        self.assertRaises(StopIteration, next, chunks)



# Copyright 2021 Red Hat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from tempest.lib import decorators

from tempest.lib.services.network import qos_limit_bandwidth_rules_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base

from oslo_log import log as logging
LOG = logging.getLogger('tempest')


class TestQosLimitBandwidthRulesClient(base.BaseServiceTest):

    FAKE_QOS_POLICY_ID = "f1011b08-1297-11e9-a1e7-c7e6825a2616"
    FAKE_MAX_BW_RULE_ID = "e758c89e-1297-11e9-a6cf-cf46a71e6699"

    FAKE_MAX_BW_RULE_REQUEST = {
        'qos_policy_id': FAKE_QOS_POLICY_ID,
        'max_kbps': 1000,
        'max_burst_kbps': 0,
        'direction': 'ingress'
    }

    FAKE_MAX_BW_RULE_RESPONSE = {
        'bandwidth_limit_rule': {
            'id': FAKE_MAX_BW_RULE_ID,
            'max_kbps': 10000,
            'max_burst_kbps': 0,
            'direction': 'ingress'
        }
    }

    FAKE_MAX_BW_RULES = {
        'bandwidth_limit_rules': [
            FAKE_MAX_BW_RULE_RESPONSE['bandwidth_limit_rule']
        ]
    }

    def setUp(self):
        super(TestQosLimitBandwidthRulesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.qos_limit_bw_client = qos_limit_bandwidth_rules_client.\
            QosLimitBandwidthRulesClient(fake_auth, "network", "regionOne")

    @decorators.idempotent_id('cde981fa-e93b-11eb-aacb-74e5f9e2a801')
    def test_create_limit_bandwidth_rules(self, bytes_body=False):
        self.check_service_client_function(
            self.qos_limit_bw_client.create_limit_bandwidth_rule,
            "tempest.lib.common.rest_client.RestClient.post",
            self.FAKE_MAX_BW_RULE_RESPONSE,
            bytes_body,
            201,
            **self.FAKE_MAX_BW_RULE_REQUEST
        )

    @decorators.idempotent_id('86e6803a-e974-11eb-aacb-74e5f9e2a801')
    def test_update_limit_bandwidth_rules(self, bytes_body=False):
        update_kwargs = {
            "max_kbps": "20000"
        }

        resp_body = {
            "bandwidth_limit_rule": copy.deepcopy(
                self.FAKE_MAX_BW_RULE_RESPONSE['bandwidth_limit_rule']
            )
        }
        resp_body["bandwidth_limit_rule"].update(update_kwargs)

        self.check_service_client_function(
            self.qos_limit_bw_client.update_limit_bandwidth_rule,
            "tempest.lib.common.rest_client.RestClient.put",
            resp_body,
            bytes_body,
            200,
            qos_policy_id=self.FAKE_QOS_POLICY_ID,
            rule_id=self.FAKE_MAX_BW_RULE_ID,
            **update_kwargs)

    @decorators.idempotent_id('be60ae6e-e979-11eb-aacb-74e5f9e2a801')
    def test_show_limit_bandwidth_rules(self, bytes_body=False):
        self.check_service_client_function(
            self.qos_limit_bw_client.show_limit_bandwidth_rule,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_MAX_BW_RULE_RESPONSE,
            bytes_body,
            200,
            qos_policy_id=self.FAKE_QOS_POLICY_ID,
            rule_id=self.FAKE_MAX_BW_RULE_ID
        )

    @decorators.idempotent_id('0a7c0964-e97b-11eb-aacb-74e5f9e2a801')
    def test_delete_limit_bandwidth_rule(self):
        self.check_service_client_function(
            self.qos_limit_bw_client.delete_limit_bandwidth_rule,
            "tempest.lib.common.rest_client.RestClient.delete",
            {},
            status=204,
            qos_policy_id=self.FAKE_QOS_POLICY_ID,
            rule_id=self.FAKE_MAX_BW_RULE_ID)

    @decorators.idempotent_id('08df88ae-e97d-11eb-aacb-74e5f9e2a801')
    def test_list_minimum_bandwidth_rules(self, bytes_body=False):
        self.check_service_client_function(
            self.qos_limit_bw_client.list_limit_bandwidth_rules,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_MAX_BW_RULES,
            bytes_body,
            200,
            qos_policy_id=self.FAKE_QOS_POLICY_ID
        )



# Copyright 2017 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib.parse import urljoin

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.volume import versions as schema
from tempest.lib.common import rest_client
from tempest.lib.services.volume import base_client


class VersionsClient(base_client.BaseClient):

    def list_versions(self):
        """List API versions

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#list-all-api-versions
        """
        version_url = self._get_base_version_url()

        resp, body = self.raw_request(version_url, 'GET')
        # NOTE: We need a raw_request() here instead of request() call because
        # "list API versions" API doesn't require an authentication and we can
        # skip it with raw_request() call.
        self._error_checker(resp, body)

        body = json.loads(body)
        self.validate_response(schema.list_versions, resp, body)
        return rest_client.ResponseBody(resp, body)

    def show_version(self, version):
        """Show API version details

        Use raw_request in order to have access to the endpoints minus
        version and project in order to add version only back.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/block-storage/v3/#show-api-v3-details
        """

        version_url = urljoin(self._get_base_version_url(), version + '/')
        headers = self.get_headers()
        headers['X-Auth-Token'] = self.token
        resp, body = self.raw_request(version_url, 'GET', headers=headers)
        self._error_checker(resp, body)
        body = json.loads(body)
        self.validate_response(schema.volume_api_version_details, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2015 GlobalLogic.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.api.network import base
from tempest.common import utils
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class SubnetPoolsTestJSON(base.BaseNetworkTest):
    """Tests the following operations in the subnetpools API:

        Create a subnet pool.
        Update a subnet pool.
        Delete a subnet pool.
        Lists subnet pool.
        Show subnet pool details.

    v2.0 of the Neutron API is assumed. It is assumed that subnet_allocation
    options mentioned in the [network-feature-enabled] section and
    default_network option mentioned in the [network] section of
    etc/tempest.conf:

    """

    @classmethod
    def skip_checks(cls):
        super(SubnetPoolsTestJSON, cls).skip_checks()
        if not utils.is_extension_enabled('subnet_allocation', 'network'):
            msg = "subnet_allocation extension not enabled."
            raise cls.skipException(msg)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('62595970-ab1c-4b7f-8fcc-fddfe55e9811')
    def test_create_list_show_update_delete_subnetpools(self):
        """Test create/list/show/update/delete of subnet pools"""
        subnetpool_name = data_utils.rand_name(
            name='subnetpools', prefix=CONF.resource_name_prefix)
        # create subnet pool
        prefix = CONF.network.default_network
        body = self.subnetpools_client.create_subnetpool(name=subnetpool_name,
                                                         prefixes=prefix)
        subnetpool_id = body["subnetpool"]["id"]
        self.addCleanup(test_utils.call_and_ignore_notfound_exc,
                        self.subnetpools_client.delete_subnetpool,
                        subnetpool_id)
        self.assertEqual(subnetpool_name, body["subnetpool"]["name"])
        # get detail about subnet pool
        body = self.subnetpools_client.show_subnetpool(subnetpool_id)
        self.assertEqual(subnetpool_name, body["subnetpool"]["name"])
        # update the subnet pool
        subnetpool_name = data_utils.rand_name(
            name='subnetpools_update', prefix=CONF.resource_name_prefix)
        body = self.subnetpools_client.update_subnetpool(subnetpool_id,
                                                         name=subnetpool_name)
        self.assertEqual(subnetpool_name, body["subnetpool"]["name"])
        # delete subnet pool
        body = self.subnetpools_client.delete_subnetpool(subnetpool_id)
        self.assertRaises(lib_exc.NotFound,
                          self.subnetpools_client.show_subnetpool,
                          subnetpool_id)



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random

import netaddr
from oslo_utils import netutils

from tempest.api.network import base
from tempest.common.utils import net_info
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class NetworksTestDHCPv6(base.BaseNetworkTest):
    _ip_version = 6

    """Test DHCPv6 specific features using SLAAC, stateless and
    stateful settings for subnets. Also it shall check dual-stack
    functionality (IPv4 + IPv6 together).
    The tests include:
        generating of SLAAC EUI-64 address in subnets with various settings
        receiving SLAAC addresses in combinations of various subnets
        receiving stateful IPv6 addresses
        addressing in subnets with router
    """

    @classmethod
    def skip_checks(cls):
        super(NetworksTestDHCPv6, cls).skip_checks()
        msg = None
        if not CONF.network_feature_enabled.ipv6:
            msg = "IPv6 is not enabled"
        elif not CONF.network_feature_enabled.ipv6_subnet_attributes:
            msg = "DHCPv6 attributes are not enabled."
        if msg:
            raise cls.skipException(msg)

    @classmethod
    def resource_setup(cls):
        super(NetworksTestDHCPv6, cls).resource_setup()
        cls.network = cls.create_network()
        cls.ports = []
        cls.subnets = []
        cls.routers = []

    def _remove_from_list_by_index(self, things_list, elem):
        for index, i in enumerate(things_list):
            if i['id'] == elem['id']:
                break
        del things_list[index]

    def _clean_network(self):
        body = self.ports_client.list_ports()
        ports = body['ports']
        for port in ports:
            if (net_info.is_router_interface_port(port) and
                port['device_id'] in [r['id'] for r in self.routers]):
                self.routers_client.remove_router_interface(port['device_id'],
                                                            port_id=port['id'])
            else:
                if port['id'] in [p['id'] for p in self.ports]:
                    self.ports_client.delete_port(port['id'])
                    self._remove_from_list_by_index(self.ports, port)
        body = self.subnets_client.list_subnets()
        subnets = body['subnets']
        for subnet in subnets:
            if subnet['id'] in [s['id'] for s in self.subnets]:
                self.subnets_client.delete_subnet(subnet['id'])
                self._remove_from_list_by_index(self.subnets, subnet)
        body = self.routers_client.list_routers()
        routers = body['routers']
        for router in routers:
            if router['id'] in [r['id'] for r in self.routers]:
                self.routers_client.delete_router(router['id'])
                self._remove_from_list_by_index(self.routers, router)

    def _get_ips_from_subnet(self, **kwargs):
        subnet = self.create_subnet(self.network, **kwargs)
        self.subnets.append(subnet)
        port_mac = data_utils.rand_mac_address()
        port = self.create_port(self.network, mac_address=port_mac)
        self.ports.append(port)
        real_ip = next(iter(port['fixed_ips']), None)['ip_address']
        eui_ip = str(netutils.get_ipv6_addr_by_EUI64(
            subnet['cidr'], port_mac))
        return real_ip, eui_ip

    @decorators.idempotent_id('e5517e62-6f16-430d-a672-f80875493d4c')
    def test_dhcpv6_stateless_eui64(self):
        """Test eui64 ip when setting slaac and statelss for subnet

        NOTE: When subnets configured with RAs SLAAC (AOM=100) and DHCP
        stateless (AOM=110) both for radvd and dnsmasq, port shall receive
        IP address calculated from its MAC.
        """
        for ra_mode, add_mode in (
                ('slaac', 'slaac'),
                ('dhcpv6-stateless', 'dhcpv6-stateless'),
        ):
            kwargs = {'ipv6_ra_mode': ra_mode,
                      'ipv6_address_mode': add_mode}
            real_ip, eui_ip = self._get_ips_from_subnet(**kwargs)
            self._clean_network()
            self.assertEqual(eui_ip, real_ip,
                             ('Real port IP is %s, but shall be %s when '
                              'ipv6_ra_mode=%s and ipv6_address_mode=%s') % (
                                 real_ip, eui_ip, ra_mode, add_mode))

    @decorators.idempotent_id('ae2f4a5d-03ff-4c42-a3b0-ce2fcb7ea832')
    def test_dhcpv6_stateless_no_ra(self):
        """Test eui64 ip when setting stateless and no radvd for subnets

        NOTE: When subnets configured with dnsmasq SLAAC and DHCP stateless
        and there is no radvd, port shall receive IP address calculated
        from its MAC and mask of subnet.
        """
        for ra_mode, add_mode in (
                (None, 'slaac'),
                (None, 'dhcpv6-stateless'),
        ):
            kwargs = {'ipv6_ra_mode': ra_mode,
                      'ipv6_address_mode': add_mode}
            kwargs = dict((k, v) for k, v in kwargs.items() if v)
            real_ip, eui_ip = self._get_ips_from_subnet(**kwargs)
            self._clean_network()
            self.assertEqual(eui_ip, real_ip,
                             ('Real port IP %s shall be equal to EUI-64 %s '
                              'when ipv6_ra_mode=%s,ipv6_address_mode=%s') % (
                                 real_ip, eui_ip,
                                 ra_mode if ra_mode else "Off",
                                 add_mode if add_mode else "Off"))

    @decorators.idempotent_id('81f18ef6-95b5-4584-9966-10d480b7496a')
    def test_dhcpv6_invalid_options(self):
        """Different configurations for radvd and dnsmasq are not allowed"""
        for ra_mode, add_mode in (
                ('dhcpv6-stateless', 'dhcpv6-stateful'),
                ('dhcpv6-stateless', 'slaac'),
                ('slaac', 'dhcpv6-stateful'),
                ('dhcpv6-stateful', 'dhcpv6-stateless'),
                ('dhcpv6-stateful', 'slaac'),
                ('slaac', 'dhcpv6-stateless'),
        ):
            kwargs = {'ipv6_ra_mode': ra_mode,
                      'ipv6_address_mode': add_mode}
            self.assertRaises(lib_exc.BadRequest,
                              self.create_subnet,
                              self.network,
                              **kwargs)

    @decorators.idempotent_id('21635b6f-165a-4d42-bf49-7d195e47342f')
    def test_dhcpv6_stateless_no_ra_no_dhcp(self):
        """Test eui64 ip when setting no radvd and no dnsmasq for subnets

        NOTE: If no radvd option and no dnsmasq option is configured
        port shall receive IP from fixed IPs list of subnet.
        """
        real_ip, eui_ip = self._get_ips_from_subnet()
        self._clean_network()
        self.assertNotEqual(eui_ip, real_ip,
                            ('Real port IP %s equal to EUI-64 %s when '
                             'ipv6_ra_mode=Off and ipv6_address_mode=Off, '
                             'but shall be taken from fixed IPs') % (
                                real_ip, eui_ip))

    @decorators.idempotent_id('4544adf7-bb5f-4bdc-b769-b3e77026cef2')
    def test_dhcpv6_two_subnets(self):
        """Test eui64 ip when creating port under network with two subnets

        NOTE: When one IPv6 subnet configured with dnsmasq SLAAC or DHCP
        stateless and other IPv6 is with DHCP stateful, port shall receive
        EUI-64 IP addresses from first subnet and DHCP address from second
        one. Order of subnet creating should be unimportant.
        """
        for order in ("slaac_first", "dhcp_first"):
            for ra_mode, add_mode in (
                    ('slaac', 'slaac'),
                    ('dhcpv6-stateless', 'dhcpv6-stateless'),
            ):
                kwargs = {'ipv6_ra_mode': ra_mode,
                          'ipv6_address_mode': add_mode}
                kwargs_dhcp = {'ipv6_address_mode': 'dhcpv6-stateful'}
                if order == "slaac_first":
                    subnet_slaac = self.create_subnet(self.network, **kwargs)
                    self.subnets.append(subnet_slaac)
                    subnet_dhcp = self.create_subnet(
                        self.network, **kwargs_dhcp)
                    self.subnets.append(subnet_dhcp)
                else:
                    subnet_dhcp = self.create_subnet(
                        self.network, **kwargs_dhcp)
                    self.subnets.append(subnet_dhcp)
                    subnet_slaac = self.create_subnet(self.network, **kwargs)
                    self.subnets.append(subnet_slaac)
                port_mac = data_utils.rand_mac_address()
                eui_ip = str(netutils.get_ipv6_addr_by_EUI64(
                    subnet_slaac['cidr'], port_mac))
                port = self.create_port(self.network, mac_address=port_mac)
                self.ports.append(port)
                real_ips = dict([(k['subnet_id'], k['ip_address'])
                                 for k in port['fixed_ips']])
                real_dhcp_ip, real_eui_ip = [real_ips[sub['id']]
                                             for sub in [subnet_dhcp,
                                                         subnet_slaac]]
                self.ports_client.delete_port(port['id'])
                self.ports.pop()
                body = self.ports_client.list_ports()
                ports_id_list = [i['id'] for i in body['ports']]
                self.assertNotIn(port['id'], ports_id_list)
                self._clean_network()
                self.assertEqual(real_eui_ip,
                                 eui_ip,
                                 'Real IP is {0}, but shall be {1}'.format(
                                     real_eui_ip,
                                     eui_ip))
                msg = ('Real IP address is {0} and it is NOT on '
                       'subnet {1}'.format(real_dhcp_ip, subnet_dhcp['cidr']))
                self.assertIn(netaddr.IPAddress(real_dhcp_ip),
                              netaddr.IPNetwork(subnet_dhcp['cidr']), msg)

    @decorators.idempotent_id('4256c61d-c538-41ea-9147-3c450c36669e')
    def test_dhcpv6_64_subnets(self):
        """Test eui64 ip when setting slaac and stateless for subnets

        NOTE: When one IPv6 subnet configured with dnsmasq SLAAC or DHCP
        stateless and other IPv4 is with DHCP of IPv4, port shall receive
        EUI-64 IP addresses from first subnet and IPv4 DHCP address from
        second one. Order of subnet creating should be unimportant.
        """
        for order in ("slaac_first", "dhcp_first"):
            for ra_mode, add_mode in (
                    ('slaac', 'slaac'),
                    ('dhcpv6-stateless', 'dhcpv6-stateless'),
            ):
                kwargs = {'ipv6_ra_mode': ra_mode,
                          'ipv6_address_mode': add_mode}
                if order == "slaac_first":
                    subnet_slaac = self.create_subnet(self.network, **kwargs)
                    self.subnets.append(subnet_slaac)
                    subnet_dhcp = self.create_subnet(
                        self.network, ip_version=4)
                    self.subnets.append(subnet_dhcp)
                else:
                    subnet_dhcp = self.create_subnet(
                        self.network, ip_version=4)
                    self.subnets.append(subnet_dhcp)
                    subnet_slaac = self.create_subnet(self.network, **kwargs)
                    self.subnets.append(subnet_slaac)
                port_mac = data_utils.rand_mac_address()
                eui_ip = str(netutils.get_ipv6_addr_by_EUI64(
                    subnet_slaac['cidr'], port_mac))
                port = self.create_port(self.network, mac_address=port_mac)
                self.ports.append(port)
                real_ips = dict([(k['subnet_id'], k['ip_address'])
                                 for k in port['fixed_ips']])
                real_dhcp_ip, real_eui_ip = [real_ips[sub['id']]
                                             for sub in [subnet_dhcp,
                                                         subnet_slaac]]
                self._clean_network()
                self.assertEqual(real_eui_ip,
                                 eui_ip,
                                 'Real IP is {0}, but shall be {1}'.format(
                                     real_eui_ip,
                                     eui_ip))
                msg = ('Real IP address is {0} and it is NOT on '
                       'subnet {1}'.format(real_dhcp_ip, subnet_dhcp['cidr']))
                self.assertIn(netaddr.IPAddress(real_dhcp_ip),
                              netaddr.IPNetwork(subnet_dhcp['cidr']), msg)

    @decorators.idempotent_id('4ab211a0-276f-4552-9070-51e27f58fecf')
    def test_dhcp_stateful(self):
        """Test creating port when setting stateful for subnets

        NOTE: With all options below, DHCPv6 shall allocate address from
        subnet pool to port.
        """
        for ra_mode, add_mode in (
                ('dhcpv6-stateful', 'dhcpv6-stateful'),
                ('dhcpv6-stateful', None),
                (None, 'dhcpv6-stateful'),
        ):
            kwargs = {'ipv6_ra_mode': ra_mode,
                      'ipv6_address_mode': add_mode}
            kwargs = dict((k, v) for k, v in kwargs.items() if v)
            subnet = self.create_subnet(self.network, **kwargs)
            self.subnets.append(subnet)
            port = self.create_port(self.network)
            self.ports.append(port)
            port_ip = next(iter(port['fixed_ips']), None)['ip_address']
            self._clean_network()
            msg = ('Real IP address is {0} and it is NOT on '
                   'subnet {1}'.format(port_ip, subnet['cidr']))
            self.assertIn(netaddr.IPAddress(port_ip),
                          netaddr.IPNetwork(subnet['cidr']), msg)

    @decorators.idempotent_id('51a5e97f-f02e-4e4e-9a17-a69811d300e3')
    def test_dhcp_stateful_fixedips(self):
        """Test creating port with fixed ip when setting stateful for subnets

        NOTE: With all options below, port shall be able to get
        requested IP from fixed IP range not depending on
        DHCP stateful (not SLAAC!) settings configured.
        """
        for ra_mode, add_mode in (
                ('dhcpv6-stateful', 'dhcpv6-stateful'),
                ('dhcpv6-stateful', None),
                (None, 'dhcpv6-stateful'),
        ):
            kwargs = {'ipv6_ra_mode': ra_mode,
                      'ipv6_address_mode': add_mode}
            kwargs = dict((k, v) for k, v in kwargs.items() if v)
            subnet = self.create_subnet(self.network, **kwargs)
            self.subnets.append(subnet)
            ip_range = netaddr.IPRange(subnet["allocation_pools"][0]["start"],
                                       subnet["allocation_pools"][0]["end"])
            ip = netaddr.IPAddress(random.randrange(ip_range.first,
                                                    ip_range.last)).format()
            port = self.create_port(self.network,
                                    fixed_ips=[{'subnet_id': subnet['id'],
                                                'ip_address': ip}])
            self.ports.append(port)
            port_ip = next(iter(port['fixed_ips']), None)['ip_address']
            self._clean_network()
            self.assertEqual(port_ip, ip,
                             ("Port IP %s is not as fixed IP from "
                              "port create request: %s") % (
                                 port_ip, ip))

    @decorators.idempotent_id('98244d88-d990-4570-91d4-6b25d70d08af')
    def test_dhcp_stateful_fixedips_outrange(self):
        """Test creating port with fixed ip that is not in the range

        NOTE: When port gets IP address from fixed IP range it
        shall be checked if it's from subnets range.
        """
        kwargs = {'ipv6_ra_mode': 'dhcpv6-stateful',
                  'ipv6_address_mode': 'dhcpv6-stateful'}
        subnet = self.create_subnet(self.network, **kwargs)
        self.subnets.append(subnet)
        ip_range = netaddr.IPRange(subnet["allocation_pools"][0]["start"],
                                   subnet["allocation_pools"][0]["end"])
        ip = netaddr.IPAddress(random.randrange(
            ip_range.last + 1, ip_range.last + 10)).format()
        self.assertRaises(lib_exc.BadRequest,
                          self.create_port,
                          self.network,
                          fixed_ips=[{'subnet_id': subnet['id'],
                                      'ip_address': ip}])

    @decorators.idempotent_id('57b8302b-cba9-4fbb-8835-9168df029051')
    def test_dhcp_stateful_fixedips_duplicate(self):
        """Test creating port with duplicate fixed ip

        NOTE: When port gets IP address from fixed IP range it
        shall be checked if it's not duplicate.
        """
        kwargs = {'ipv6_ra_mode': 'dhcpv6-stateful',
                  'ipv6_address_mode': 'dhcpv6-stateful'}
        subnet = self.create_subnet(self.network, **kwargs)
        self.subnets.append(subnet)
        ip_range = netaddr.IPRange(subnet["allocation_pools"][0]["start"],
                                   subnet["allocation_pools"][0]["end"])
        ip = netaddr.IPAddress(random.randrange(
            ip_range.first, ip_range.last)).format()
        port = self.create_port(self.network,
                                fixed_ips=[
                                    {'subnet_id': subnet['id'],
                                     'ip_address': ip}])
        self.ports.append(port)
        self.assertRaisesRegex(lib_exc.Conflict,
                               "IpAddressAlreadyAllocated|IpAddressInUse",
                               self.create_port,
                               self.network,
                               fixed_ips=[{'subnet_id': subnet['id'],
                                           'ip_address': ip}])

    def _create_subnet_router(self, kwargs):
        subnet = self.create_subnet(self.network, **kwargs)
        self.subnets.append(subnet)
        router = self.create_router(admin_state_up=True)
        self.routers.append(router)
        port = self.create_router_interface(router['id'],
                                            subnet['id'])
        body = self.ports_client.show_port(port['port_id'])
        return subnet, body['port']

    @decorators.idempotent_id('e98f65db-68f4-4330-9fea-abd8c5192d4d')
    def test_dhcp_stateful_router(self):
        """Test creating router with dhcp stateful

        NOTE: With all options below the router interface shall
        receive DHCPv6 IP address from allocation pool.
        """
        for ra_mode, add_mode in (
                ('dhcpv6-stateful', 'dhcpv6-stateful'),
                ('dhcpv6-stateful', None),
        ):
            kwargs = {'ipv6_ra_mode': ra_mode,
                      'ipv6_address_mode': add_mode}
            kwargs = dict((k, v) for k, v in kwargs.items() if v)
            subnet, port = self._create_subnet_router(kwargs)
            port_ip = next(iter(port['fixed_ips']), None)['ip_address']
            self._clean_network()
            self.assertEqual(port_ip, subnet['gateway_ip'],
                             ("Port IP %s is not as first IP from "
                              "subnets allocation pool: %s") % (
                                 port_ip, subnet['gateway_ip']))

    def tearDown(self):
        self._clean_network()
        super(NetworksTestDHCPv6, self).tearDown()



# Copyright 2021 Red Hat, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import base


class LogResourceClient(base.BaseNetworkClient):

    def create_log(self, **kwargs):
        """Creates a log resource.

        Creates a log resource by using the configuration that you define in
        the request object.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#create-log
        """
        uri = '/log/logs/'
        post_data = {'log': kwargs}
        return self.create_resource(uri, post_data)

    def update_log(self, log_id, **kwargs):
        """Updates a log resource.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#update-log
        """
        uri = '/log/logs/%s' % log_id
        post_data = {'log': kwargs}
        return self.update_resource(uri, post_data)

    def show_log(self, log_id, **fields):
        """Shows details for a log id.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#show-log
        """
        uri = '/log/logs/%s' % log_id
        return self.show_resource(uri, **fields)

    def delete_log(self, log_id):
        """Deletes a log resource.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#delete-log
        """
        uri = '/log/logs/%s' % log_id
        return self.delete_resource(uri)

    def list_logs(self, **filters):
        """Lists Logs.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/network/v2/index.html#list-logs
        """
        uri = '/log/logs'
        return self.list_resources(uri, **filters)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class ImageMembersClient(rest_client.RestClient):
    api_version = "v2"

    def list_image_members(self, image_id):
        """List image members.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#list-image-members
        """
        url = 'images/%s/members' % image_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_image_member(self, image_id, **kwargs):
        """Create an image member.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#create-image-member
        """
        url = 'images/%s/members' % image_id
        data = json.dumps(kwargs)
        resp, body = self.post(url, data)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def update_image_member(self, image_id, member_id, **kwargs):
        """Update an image member.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#update-image-member
        """
        url = 'images/%s/members/%s' % (image_id, member_id)
        data = json.dumps(kwargs)
        resp, body = self.put(url, data)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_image_member(self, image_id, member_id):
        """Show an image member.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#show-image-member-details
        """
        url = 'images/%s/members/%s' % (image_id, member_id)
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, json.loads(body))

    def delete_image_member(self, image_id, member_id):
        """Delete an image member.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#delete-image-member
        """
        url = 'images/%s/members/%s' % (image_id, member_id)
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)



# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock

from oslo_serialization import jsonutils as json

from tempest.lib.services.network import base as network_base
from tempest.lib.services.network import security_groups_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSecurityGroupsClient(base.BaseServiceTest):

    FAKE_SEC_GROUP_ID = "85cc3048-abc3-43cc-89b3-377341426ac5"

    FAKE_SECURITY_GROUPS = {
        "security_groups": [
            {
                "description": "default",
                "id": FAKE_SEC_GROUP_ID,
                "name": "fake-security-group-name",
                "security_group_rules": [
                    {
                        "direction": "egress",
                        "ethertype": "IPv4",
                        "id": "38ce2d8e-e8f1-48bd-83c2-d33cb9f50c3d",
                        "port_range_max": None,
                        "port_range_min": None,
                        "protocol": None,
                        "remote_group_id": None,
                        "remote_ip_prefix": None,
                        "security_group_id": FAKE_SEC_GROUP_ID,
                        "project_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                        "tenant_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                        "description": ""
                    },
                    {
                        "direction": "egress",
                        "ethertype": "IPv6",
                        "id": "565b9502-12de-4ffd-91e9-68885cff6ae1",
                        "port_range_max": None,
                        "port_range_min": None,
                        "protocol": None,
                        "remote_group_id": None,
                        "remote_ip_prefix": None,
                        "security_group_id": FAKE_SEC_GROUP_ID,
                        "project_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                        "tenant_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                        "description": ""
                    }
                ],
                "project_id": "e4f50856753b4dc6afee5fa6b9b6c550",
                "tenant_id": "e4f50856753b4dc6afee5fa6b9b6c550"
            }
        ]
    }

    FAKE_SECURITY_GROUP = {
        "security_group": copy.deepcopy(
            FAKE_SECURITY_GROUPS["security_groups"][0])
    }

    def setUp(self):
        super(TestSecurityGroupsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = security_groups_client.SecurityGroupsClient(
            fake_auth, 'network', 'regionOne')

    def _test_list_security_groups(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_security_groups,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SECURITY_GROUPS,
            bytes_body,
            mock_args='v2.0/security-groups')

    def _test_create_security_group(self, bytes_body=False):
        kwargs = {'name': 'fake-security-group-name'}
        payload = json.dumps({"security_group": kwargs}, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(network_base.json, 'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.create_security_group,
                'tempest.lib.common.rest_client.RestClient.post',
                self.FAKE_SECURITY_GROUP,
                bytes_body,
                status=201,
                mock_args=['v2.0/security-groups', payload],
                **kwargs)

    def _test_show_security_group(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_security_group,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SECURITY_GROUP,
            bytes_body,
            security_group_id=self.FAKE_SEC_GROUP_ID,
            mock_args='v2.0/security-groups/%s' % self.FAKE_SEC_GROUP_ID)

    def _test_update_security_group(self, bytes_body=False):
        kwargs = {'name': 'updated-security-group-name'}
        resp_body = copy.deepcopy(self.FAKE_SECURITY_GROUP)
        resp_body["security_group"]["name"] = 'updated-security-group-name'

        payload = json.dumps({'security_group': kwargs}, sort_keys=True)
        json_dumps = json.dumps

        # NOTE: Use sort_keys for json.dumps so that the expected and actual
        # payloads are guaranteed to be identical for mock_args assert check.
        with mock.patch.object(network_base.json, 'dumps') as mock_dumps:
            mock_dumps.side_effect = lambda d: json_dumps(d, sort_keys=True)

            self.check_service_client_function(
                self.client.update_security_group,
                'tempest.lib.common.rest_client.RestClient.put',
                resp_body,
                bytes_body,
                security_group_id=self.FAKE_SEC_GROUP_ID,
                mock_args=['v2.0/security-groups/%s' % self.FAKE_SEC_GROUP_ID,
                           payload],
                **kwargs)

    def test_list_security_groups_with_str_body(self):
        self._test_list_security_groups()

    def test_list_security_groups_with_bytes_body(self):
        self._test_list_security_groups(bytes_body=True)

    def test_create_security_group_with_str_body(self):
        self._test_create_security_group()

    def test_create_security_group_with_bytes_body(self):
        self._test_create_security_group(bytes_body=True)

    def test_show_security_group_with_str_body(self):
        self._test_show_security_group()

    def test_show_security_group_with_bytes_body(self):
        self._test_show_security_group(bytes_body=True)

    def test_update_security_group_with_str_body(self):
        self._test_update_security_group()

    def test_update_security_group_with_bytes_body(self):
        self._test_update_security_group(bytes_body=True)

    def test_delete_security_group(self):
        self.check_service_client_function(
            self.client.delete_security_group,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            security_group_id=self.FAKE_SEC_GROUP_ID,
            mock_args='v2.0/security-groups/%s' % self.FAKE_SEC_GROUP_ID)



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

list_services = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'services': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'binary': {'type': 'string'},
                        'disabled_reason': {'type': ['string', 'null']},
                        'host': {'type': 'string'},
                        'state': {'enum': ['up', 'down']},
                        'status': {'enum': ['enabled', 'disabled']},
                        'frozen': {'type': 'boolean'},
                        'updated_at': parameter_types.date_time,
                        'zone': {'type': 'string'},
                        'replication_status': {'type': 'string'},
                        'active_backend_id': {'type': ['string', 'null']},
                        'backend_state': {'type': 'string'},
                    },
                    'additionalProperties': False,
                    'required': ['binary', 'disabled_reason', 'host', 'state',
                                 'status', 'updated_at', 'zone']
                }
            }
        },
        'additionalProperties': False,
        'required': ['services']
    }
}

enable_service = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'disabled': {'type': 'boolean'},
            'status': {'enum': ['enabled', 'disabled']},
            'host': {'type': 'string'},
            'service': {'type': 'string'},
            'binary': {'type': 'string'},
            'disabled_reason': {'type': ['string', 'null']}
        },
        'additionalProperties': False,
        'required': ['disabled', 'status', 'host', 'service',
                     'binary', 'disabled_reason']
    }
}

disable_service = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'disabled': {'type': 'boolean'},
            'status': {'enum': ['enabled', 'disabled']},
            'host': {'type': 'string'},
            'service': {'type': 'string'},
            'binary': {'type': 'string'},
        },
        'additionalProperties': False,
        'required': ['disabled', 'status', 'host', 'service', 'binary']
    }
}

disable_log_reason = copy.deepcopy(enable_service)

freeze_host = {'status_code': [200]}
thaw_host = {'status_code': [200]}



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

attachments = {
    'type': 'array',
    'items': {
        'type': 'object',
        'properties': {
            'server_id': {'type': ['string', 'null'], 'format': 'uuid'},
            'attachment_id': {'type': 'string', 'format': 'uuid'},
            'attached_at': parameter_types.date_time_or_null,
            'host_name': {'type': ['string', 'null']},
            'volume_id': {'type': 'string', 'format': 'uuid'},
            'device': {'type': ['string', 'null']},
            'id': {'type': 'string', 'format': 'uuid'}
        },
        'additionalProperties': False,
        'required': ['server_id', 'attachment_id', 'host_name',
                     'volume_id', 'device', 'id']
    }
}

common_show_volume = {
    'type': 'object',
    'properties': {
        'migration_status': {'type': ['string', 'null']},
        'attachments': attachments,
        'links': parameter_types.links,
        'availability_zone': {'type': ['string', 'null']},
        'os-vol-host-attr:host': {
            'type': ['string', 'null'], 'pattern': '.+@.+#.+'},
        'encrypted': {'type': 'boolean'},
        'updated_at': parameter_types.date_time_or_null,
        'replication_status': {'type': ['string', 'null']},
        'snapshot_id': parameter_types.uuid_or_null,
        'id': {'type': 'string', 'format': 'uuid'},
        'size': {'type': 'integer'},
        'user_id': {'type': 'string', 'format': 'uuid'},
        'os-vol-tenant-attr:tenant_id': {'type': 'string',
                                         'format': 'uuid'},
        'os-vol-mig-status-attr:migstat': {'type': ['string', 'null']},
        'metadata': {'type': 'object'},
        'status': {'type': 'string'},
        'volume_image_metadata': {'type': ['object', 'null']},
        'description': {'type': ['string', 'null']},
        'multiattach': {'type': 'boolean'},
        'source_volid': parameter_types.uuid_or_null,
        'consistencygroup_id': parameter_types.uuid_or_null,
        'os-vol-mig-status-attr:name_id': parameter_types.uuid_or_null,
        'name': {'type': ['string', 'null']},
        'bootable': {'type': 'string'},
        'created_at': parameter_types.date_time,
        'volume_type': {'type': ['string', 'null']},
        # TODO(zhufl): group_id is added in 3.13, we should move it to the
        # 3.13 schema file when microversion is supported in volume interfaces
        'group_id': parameter_types.uuid_or_null,
        # TODO(zhufl): provider_id is added in 3.21, we should move it to the
        # 3.21 schema file when microversion is supported in volume interfaces
        'provider_id': parameter_types.uuid_or_null,
        # TODO(zhufl): service_uuid and shared_targets are added in 3.48,
        # we should move them to the 3.48 schema file when microversion
        # is supported in volume interfaces.
        'service_uuid': parameter_types.uuid_or_null,
        'shared_targets': {'type': 'boolean'}
    },
    'additionalProperties': False,
    'required': ['attachments', 'links', 'encrypted',
                 'updated_at', 'replication_status', 'id',
                 'size', 'user_id', 'availability_zone',
                 'metadata', 'status', 'description',
                 'multiattach', 'consistencygroup_id',
                 'name', 'bootable', 'created_at',
                 'volume_type', 'snapshot_id', 'source_volid']
}

list_volumes_no_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'volumes': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'links': parameter_types.links,
                        'id': {'type': 'string', 'format': 'uuid'},
                        'name': {'type': ['string', 'null']},
                        # TODO(zhufl): count is added in 3.45, we should move
                        # it to the 3.45 schema file when microversion is
                        # supported in volume interfaces
                        # 'count': {'type': 'integer'}
                    },
                    'additionalProperties': False,
                    'required': ['links', 'id', 'name']
                }
            },
            'volumes_links': parameter_types.links
        },
        'additionalProperties': False,
        'required': ['volumes']
    }
}

show_volume = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'volume': common_show_volume
        },
        'additionalProperties': False,
        'required': ['volume']
    }
}

list_volumes_detail = copy.deepcopy(common_show_volume)
# TODO(zhufl): count is added in 3.45, we should move it to the 3.45 schema
# file when microversion is supported in volume interfaces
# list_volumes_detail['properties'].update({'count': {'type': 'integer'}})
list_volumes_with_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'volumes': {
                'type': 'array',
                'items': list_volumes_detail
            },
            'volumes_links': parameter_types.links
        },
        'additionalProperties': False,
        'required': ['volumes']
    }
}

create_volume = {
    'status_code': [202],
    'response_body': {
        'type': 'object',
        'properties': {
            'volume': {
                'type': 'object',
                'properties': {
                    'migration_status': {'type': ['string', 'null']},
                    'attachments': attachments,
                    'links': parameter_types.links,
                    'availability_zone': {'type': ['string', 'null']},
                    'encrypted': {'type': 'boolean'},
                    'updated_at': parameter_types.date_time_or_null,
                    'replication_status': {'type': ['string', 'null']},
                    'snapshot_id': parameter_types.uuid_or_null,
                    'id': {'type': 'string', 'format': 'uuid'},
                    'size': {'type': 'integer'},
                    'user_id': {'type': 'string', 'format': 'uuid'},
                    'metadata': {'type': 'object'},
                    'status': {'type': 'string'},
                    'description': {'type': ['string', 'null']},
                    'multiattach': {'type': 'boolean'},
                    'source_volid': parameter_types.uuid_or_null,
                    'consistencygroup_id': parameter_types.uuid_or_null,
                    'name': {'type': ['string', 'null']},
                    'bootable': {'type': 'string'},
                    'created_at': parameter_types.date_time,
                    'volume_type': {'type': ['string', 'null']},
                    # TODO(zhufl): group_id is added in 3.13, we should move
                    # it to the 3.13 schema file when microversion is
                    # supported in volume interfaces.
                    'group_id': parameter_types.uuid_or_null,
                    # TODO(zhufl): provider_id is added in 3.21, we should
                    # move it to the 3.21 schema file when microversion is
                    # supported in volume interfaces
                    'provider_id': parameter_types.uuid_or_null,
                    # TODO(zhufl): service_uuid and shared_targets are added
                    # in 3.48, we should move them to the 3.48 schema file
                    # when microversion is supported in volume interfaces.
                    'service_uuid': parameter_types.uuid_or_null,
                    'shared_targets': {'type': 'boolean'}
                },
                'additionalProperties': False,
                'required': ['attachments', 'links', 'encrypted',
                             'updated_at', 'replication_status', 'id',
                             'size', 'user_id', 'availability_zone',
                             'metadata', 'status', 'description',
                             'multiattach', 'consistencygroup_id',
                             'name', 'bootable', 'created_at',
                             'volume_type', 'snapshot_id', 'source_volid']
            }
        },
        'additionalProperties': False,
        'required': ['volume']
    }
}

update_volume = copy.deepcopy(create_volume)
update_volume.update({'status_code': [200]})

delete_volume = {'status_code': [202]}

show_volume_summary = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'volume-summary': {
                'type': 'object',
                'properties': {
                    'total_size': {'type': 'integer'},
                    'total_count': {'type': 'integer'},
                    # TODO(zhufl): metadata is added in 3.36, we should move
                    # it to the 3.36 schema file when microversion is
                    # supported in volume interfaces
                    'metadata': {'type': 'object'},
                },
                'additionalProperties': False,
                'required': ['total_size', 'total_count']
            }
        },
        'additionalProperties': False,
        'required': ['volume-summary']
    }
}

# TODO(zhufl): This is under discussion, so will be merged in a separate patch.
# https://bugs.launchpad.net/cinder/+bug/1880566
# upload_volume = {
#     'status_code': [202],
#     'response_body': {
#         'type': 'object',
#         'properties': {
#             'os-volume_upload_image': {
#                 'type': 'object',
#                 'properties': {
#                     'status': {'type': 'string'},
#                     'image_name': {'type': 'string'},
#                     'disk_format': {'type': 'string'},
#                     'container_format': {'type': 'string'},
#                     'is_public': {'type': 'boolean'},
#                     'visibility': {'type': 'string'},
#                     'protected': {'type': 'boolean'},
#                     'updated_at': parameter_types.date_time_or_null,
#                     'image_id': {'type': 'string', 'format': 'uuid'},
#                     'display_description': {'type': ['string', 'null']},
#                     'id': {'type': 'string', 'format': 'uuid'},
#                     'size': {'type': 'integer'},
#                     'volume_type': {
#                         'type': ['object', 'null'],
#                         'properties': {
#                             'created_at': parameter_types.date_time,
#                             'deleted': {'type': 'boolean'},
#                             'deleted_at': parameter_types.date_time_or_null,
#                             'description': {'type': ['string', 'null']},
#                             'extra_specs': {
#                                 'type': 'object',
#                                 'patternProperties': {
#                                     '^.+$': {'type': 'string'}
#                                 }
#                             },
#                             'id': {'type': 'string', 'format': 'uuid'},
#                             'is_public': {'type': 'boolean'},
#                             'name': {'type': ['string', 'null']},
#                             'qos_specs_id': parameter_types.uuid_or_null,
#                             'updated_at': parameter_types.date_time_or_null
#                         },
#                     }
#                 },
#                 'additionalProperties': False,
#                 'required': ['status', 'image_name', 'updated_at',
#                              'image_id',
#                              'display_description', 'id', 'size',
#                              'volume_type', 'disk_format',
#                              'container_format']
#             }
#         },
#         'additionalProperties': False,
#         'required': ['os-volume_upload_image']
#     }
# }

attach_volume = {'status_code': [202]}
set_bootable_volume = {'status_code': [200]}
detach_volume = {'status_code': [202]}
terminate_connection = {'status_code': [202]}
reserve_volume = {'status_code': [202]}
unreserve_volume = {'status_code': [202]}
extend_volume = {'status_code': [202]}
reset_volume_status = {'status_code': [202]}
update_volume_readonly = {'status_code': [202]}
force_delete_volume = {'status_code': [202]}
retype_volume = {'status_code': [202]}
force_detach_volume = {'status_code': [202]}

create_volume_metadata = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'metadata': {'type': 'object'},
        },
        'additionalProperties': False,
        'required': ['metadata']
    }
}

show_volume_metadata = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'metadata': {'type': 'object'},
        },
        'additionalProperties': False,
        'required': ['metadata']
    }
}
update_volume_metadata = copy.deepcopy(show_volume_metadata)

show_volume_metadata_item = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'meta': {'type': 'object'},
        },
        'additionalProperties': False,
        'required': ['meta']
    }
}
update_volume_metadata_item = copy.deepcopy(show_volume_metadata_item)
delete_volume_metadata_item = {'status_code': [200]}

update_volume_image_metadata = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {'metadata': {'type': 'object'}},
        'additionalProperties': False,
        'required': ['metadata']
    }
}
delete_volume_image_metadata = {'status_code': [200]}
show_volume_image_metadata = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'metadata': {'type': 'object'},
        },
        'additionalProperties': False,
        'required': ['metadata']
    }
}

unmanage_volume = {'status_code': [202]}



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types
from tempest.lib.api_schema.response.volume import backups

# Volume micro version 3.64:
# 1. Include the encryption_key_id in volume and backup
# details when the associated volume is encrypted.
# https://docs.openstack.org/cinder/latest/contributor/
# api_microversion_history.html

common_show_backup = copy.deepcopy(backups.common_show_backup)
common_show_backup['properties'].update(
    {'encryption_key_id': parameter_types.uuid_or_null})

create_backup = copy.deepcopy(backups.create_backup)
update_backup = copy.deepcopy(backups.update_backup)
restore_backup = copy.deepcopy(backups.restore_backup)
delete_backup = copy.deepcopy(backups.delete_backup)
# show backup refers to common_show_backup
show_backup = copy.deepcopy(backups.show_backup)
show_backup['response_body']['properties']['backup'] = common_show_backup
list_backups_no_detail = copy.deepcopy(backups.list_backups_no_detail)
# list_backups_detail refers to latest common_show_backup
list_backups_detail = copy.deepcopy(common_show_backup)
list_backups_detail['properties'].update({'count': {'type': 'integer'}})
list_backups_with_detail = copy.deepcopy(backups.list_backups_with_detail)
# list_backups_with_detail refers to latest list_backups_detail
list_backups_with_detail['response_body']['properties']['backups']['items'] =\
    list_backups_detail
export_backup = copy.deepcopy(backups.export_backup)
import_backup = copy.deepcopy(backups.import_backup)
reset_backup_status = copy.deepcopy(backups.reset_backup_status)



# Copyright 2014 NEC Corporation.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import migrations as schema
from tempest.lib.api_schema.response.compute.v2_23 import migrations \
    as schemav223
from tempest.lib.api_schema.response.compute.v2_59 import migrations \
    as schemav259
from tempest.lib.api_schema.response.compute.v2_80 import migrations \
    as schemav280
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class MigrationsClient(base_compute_client.BaseComputeClient):
    schema_versions_info = [
        {'min': None, 'max': '2.22', 'schema': schema},
        {'min': '2.23', 'max': '2.58', 'schema': schemav223},
        {'min': '2.59', 'max': '2.79', 'schema': schemav259},
        {'min': '2.80', 'max': None, 'schema': schemav280}]

    def list_migrations(self, **params):
        """List all migrations.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/compute/#list-migrations
        """

        url = 'os-migrations'
        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        body = json.loads(body)
        schema = self.get_schema(self.schema_versions_info)
        self.validate_response(schema.list_migrations, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2016 Ericsson India Global Services Private Limited
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest.lib import decorators


class MetadataResourceTypesTest(base.BaseV2ImageAdminTest):
    """Test the Metadata definition resource types basic functionality"""

    @decorators.idempotent_id('6f358a4e-5ef0-11e6-a795-080027d0d606')
    def test_basic_meta_def_resource_type_association(self):
        """Test image resource type associations"""
        # Get the available resource types and use one resource_type
        body = self.resource_types_client.list_resource_types()
        resource_name = body['resource_types'][0]['name']
        # Create a namespace
        namespace = self.create_namespace()
        # Create resource type association
        body = self.resource_types_client.create_resource_type_association(
            namespace['namespace'], name=resource_name)
        self.assertEqual(body['name'], resource_name)
        # NOTE(raiesmh08): Here intentionally I have not added addcleanup
        # method for resource type dissociation because its a metadata add and
        # being cleaned as soon as namespace is cleaned at test case level.
        # When namespace cleans, resource type association will automatically
        # clean without any error or dependency.

        # List resource type associations and validate creation
        rs_type_associations = [
            rs_type_association['name'] for rs_type_association in
            self.resource_types_client.list_resource_type_association(
                namespace['namespace'])['resource_type_associations']]
        self.assertIn(resource_name, rs_type_associations)
        # Delete resource type association
        self.resource_types_client.delete_resource_type_association(
            namespace['namespace'], resource_name)
        # List resource type associations and validate deletion
        rs_type_associations = [
            rs_type_association['name'] for rs_type_association in
            self.resource_types_client.list_resource_type_association(
                namespace['namespace'])['resource_type_associations']]
        self.assertNotIn(resource_name, rs_type_associations)



# Copyright 2017 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.identity.v3 import versions_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestIdentityClient(base.BaseServiceTest):

    FAKE_VERSIONS_INFO = {
        "versions": {
            "values": [
                {"status": "stable", "updated": "2017-02-22T00:00:00Z",
                 "media-types": [
                     {"base": "application/json", "type":
                      "application/vnd.openstack.identity-v3+json"}
                 ],
                 "id": "v3.8",
                 "links": [
                     {"href": "https://15.184.67.226/identity_admin/v3/",
                      "rel": "self"}
                 ]},
                {"status": "deprecated", "updated": "2016-08-04T00:00:00Z",
                 "media-types": [
                     {"base": "application/json",
                      "type": "application/vnd.openstack.identity-v2.0+json"}
                 ],
                 "id": "v2.0",
                 "links": [
                     {"href": "https://15.184.67.226/identity_admin/v2.0/",
                      "rel": "self"},
                     {"href": "https://docs.openstack.org/",
                      "type": "text/html", "rel": "describedby"}
                 ]}
            ]
        }
    }

    def setUp(self):
        super(TestIdentityClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = versions_client.VersionsClient(fake_auth,
                                                     'identity',
                                                     'regionOne')

    def _test_list_versions(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_versions,
            'tempest.lib.common.rest_client.RestClient.raw_request',
            self.FAKE_VERSIONS_INFO,
            bytes_body,
            300)

    def test_list_versions_with_str_body(self):
        self._test_list_versions()

    def test_list_versions_with_bytes_body(self):
        self._test_list_versions(bytes_body=True)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import copy

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types
from tempest.lib.api_schema.response.compute.v2_1 import servers

# Compute microversion 2.3:
# 1. New attributes in 'os-extended-volumes:volumes_attached' dict.
#      'delete_on_termination'
# 2. New attributes in 'server' dict.
#      'OS-EXT-SRV-ATTR:reservation_id'
#      'OS-EXT-SRV-ATTR:launch_index'
#      'OS-EXT-SRV-ATTR:kernel_id'
#      'OS-EXT-SRV-ATTR:ramdisk_id'
#      'OS-EXT-SRV-ATTR:hostname'
#      'OS-EXT-SRV-ATTR:root_device_name'
#      'OS-EXT-SRV-ATTR:user_data'

server_detail = {
    'type': 'object',
    'properties': {
        'id': {'type': 'string'},
        'name': {'type': 'string'},
        'status': {'type': 'string'},
        'image': {'oneOf': [
            {'type': 'object',
                'properties': {
                    'id': {'type': 'string'},
                    'links': parameter_types.links
                },
                'additionalProperties': False,
                'required': ['id', 'links']},
            {'type': ['string', 'null']}
        ]},
        'flavor': {
            'type': 'object',
            'properties': {
                'id': {'type': 'string'},
                'links': parameter_types.links
            },
            'additionalProperties': False,
            'required': ['id', 'links']
        },
        'fault': {
            'type': 'object',
            'properties': {
                'code': {'type': 'integer'},
                'created': {'type': 'string'},
                'message': {'type': 'string'},
                'details': {'type': 'string'},
            },
            'additionalProperties': False,
            # NOTE(gmann): 'details' is not necessary to be present
            #  in the 'fault'. So it is not defined as 'required'.
            'required': ['code', 'created', 'message']
        },
        'user_id': {'type': 'string'},
        'tenant_id': {'type': 'string'},
        'created': {'type': 'string'},
        'updated': {'type': 'string'},
        'progress': {'type': 'integer'},
        'metadata': {'type': 'object'},
        'links': parameter_types.links,
        'addresses': parameter_types.addresses,
        'hostId': {'type': 'string'},
        'OS-DCF:diskConfig': {'type': 'string'},
        'accessIPv4': parameter_types.access_ip_v4,
        'accessIPv6': parameter_types.access_ip_v6,
        'key_name': {'type': ['string', 'null']},
        'security_groups': {'type': 'array'},
        'OS-SRV-USG:launched_at': {'type': ['string', 'null']},
        'OS-SRV-USG:terminated_at': {'type': ['string', 'null']},
        'OS-EXT-AZ:availability_zone': {'type': 'string'},
        'OS-EXT-STS:task_state': {'type': ['string', 'null']},
        'OS-EXT-STS:vm_state': {'type': 'string'},
        'OS-EXT-STS:power_state': parameter_types.power_state,
        'OS-EXT-SRV-ATTR:host': {'type': ['string', 'null']},
        'OS-EXT-SRV-ATTR:instance_name': {'type': 'string'},
        'OS-EXT-SRV-ATTR:hypervisor_hostname': {'type': ['string', 'null']},
        'config_drive': {'type': 'string'},
        # NOTE(gmann): new attributes in version 2.3
        'os-extended-volumes:volumes_attached': {
            'type': 'array',
            'items': {
                'type': 'object',
                'properties': {
                    'id': {'type': 'string'},
                    'delete_on_termination': {'type': 'boolean'}
                },
                'additionalProperties': False,
            },
        },
        'OS-EXT-SRV-ATTR:reservation_id': {'type': ['string', 'null']},
        'OS-EXT-SRV-ATTR:launch_index': {'type': 'integer'},
        'OS-EXT-SRV-ATTR:kernel_id': {'type': ['string', 'null']},
        'OS-EXT-SRV-ATTR:ramdisk_id': {'type': ['string', 'null']},
        'OS-EXT-SRV-ATTR:hostname': {'type': 'string'},
        'OS-EXT-SRV-ATTR:root_device_name': {'type': ['string', 'null']},
        'OS-EXT-SRV-ATTR:user_data': {'type': ['string', 'null']},
    },
    'additionalProperties': False,
    # NOTE(gmann): 'progress' attribute is present in the response
    # only when server's status is one of the progress statuses
    # ("ACTIVE","BUILD", "REBUILD", "RESIZE","VERIFY_RESIZE")
    # 'fault' attribute is present in the response
    # only when server's status is one of the  "ERROR", "DELETED".
    # OS-DCF:diskConfig and accessIPv4/v6 are API
    # extensions, and some environments return a response
    # without these attributes.So these are not defined as 'required'.
    'required': ['id', 'name', 'status', 'image', 'flavor',
                 'user_id', 'tenant_id', 'created', 'updated',
                 'metadata', 'links', 'addresses', 'hostId']
}

server_detail['properties']['addresses']['patternProperties'][
    '^[a-zA-Z0-9-_.]+$']['items']['properties'].update({
        'OS-EXT-IPS:type': {'type': 'string'},
        'OS-EXT-IPS-MAC:mac_addr': parameter_types.mac_address})
# NOTE(gmann): Update OS-EXT-IPS:type and OS-EXT-IPS-MAC:mac_addr
# attributes in server address. Those are API extension,
# and some environments return a response without
# these attributes. So they are not 'required'.

get_server = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'server': server_detail
        },
        'additionalProperties': False,
        'required': ['server']
    }
}

list_servers_detail = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'servers': {
                'type': 'array',
                'items': server_detail
            },
            'servers_links': parameter_types.links
        },
        'additionalProperties': False,
        # NOTE(gmann): servers_links attribute is not necessary to be
        # present always So it is not 'required'.
        'required': ['servers']
    }
}

# NOTE: Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.1 ***
list_servers = copy.deepcopy(servers.list_servers)
update_server = copy.deepcopy(servers.update_server)
rebuild_server = copy.deepcopy(servers.rebuild_server)
rebuild_server_with_admin_pass = copy.deepcopy(
    servers.rebuild_server_with_admin_pass)
show_server_diagnostics = copy.deepcopy(servers.show_server_diagnostics)
attach_volume = copy.deepcopy(servers.attach_volume)
show_volume_attachment = copy.deepcopy(servers.show_volume_attachment)
list_volume_attachments = copy.deepcopy(servers.list_volume_attachments)
show_instance_action = copy.deepcopy(servers.show_instance_action)
create_backup = copy.deepcopy(servers.create_backup)



# Copyright 2015 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import hypervisor_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestHypervisorClient(base.BaseServiceTest):

    hypervisor_id = "1"
    hypervisor_name = "hyper.hostname.com"

    def setUp(self):
        super(TestHypervisorClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = hypervisor_client.HypervisorClient(
            fake_auth, 'compute', 'regionOne')

    def test_list_hypervisor_str_body(self):
        self._test_list_hypervisor(bytes_body=False)

    def test_list_hypervisor_byte_body(self):
        self._test_list_hypervisor(bytes_body=True)

    def _test_list_hypervisor(self, bytes_body=False):
        expected = {"hypervisors": [{
            "id": 1,
            "hypervisor_hostname": "hypervisor1.hostname.com"},
            {
            "id": 2,
            "hypervisor_hostname": "hypervisor2.hostname.com"}]}
        self.check_service_client_function(
            self.client.list_hypervisors,
            'tempest.lib.common.rest_client.RestClient.get',
            expected, bytes_body)

    def test_show_hypervisor_str_body(self):
        self._test_show_hypervisor(bytes_body=False)

    def test_show_hypervisor_byte_body(self):
        self._test_show_hypervisor(bytes_body=True)

    def _test_show_hypervisor(self, bytes_body=False):
        expected = {"hypervisor": {
            "cpu_info": "?",
            "current_workload": 0,
            "disk_available_least": 1,
            "host_ip": "10.10.10.10",
            "free_disk_gb": 1028,
            "free_ram_mb": 7680,
            "hypervisor_hostname": "fake-mini",
            "hypervisor_type": "fake",
            "hypervisor_version": 1,
            "id": 1,
            "local_gb": 1028,
            "local_gb_used": 0,
            "memory_mb": 8192,
            "memory_mb_used": 512,
            "running_vms": 0,
            "service": {
                "host": "fake_host",
                "id": 2},
            "vcpus": 1,
            "vcpus_used": 0}}
        self.check_service_client_function(
            self.client.show_hypervisor,
            'tempest.lib.common.rest_client.RestClient.get',
            expected, bytes_body,
            hypervisor_id=self.hypervisor_id)

    def test_list_servers_on_hypervisor_str_body(self):
        self._test_list_servers_on_hypervisor(bytes_body=False)

    def test_list_servers_on_hypervisor_byte_body(self):
        self._test_list_servers_on_hypervisor(bytes_body=True)

    def _test_list_servers_on_hypervisor(self, bytes_body=False):
        expected = {"hypervisors": [{
            "id": 1,
            "hypervisor_hostname": "hyper.hostname.com",
            "servers": [{
                "uuid": "e1ae8fc4-b72d-4c2f-a427-30dd420b6277",
                "name": "instance-00000001"},
                {
                "uuid": "e1ae8fc4-b72d-4c2f-a427-30dd42066666",
                "name": "instance-00000002"}
                ]}
            ]}
        self.check_service_client_function(
            self.client.list_servers_on_hypervisor,
            'tempest.lib.common.rest_client.RestClient.get',
            expected, bytes_body,
            hypervisor_name=self.hypervisor_name)

    def test_show_hypervisor_statistics_str_body(self):
        self._test_show_hypervisor_statistics(bytes_body=False)

    def test_show_hypervisor_statistics_byte_body(self):
        self._test_show_hypervisor_statistics(bytes_body=True)

    def _test_show_hypervisor_statistics(self, bytes_body=False):
        expected = {
            "hypervisor_statistics": {
                "count": 1,
                "current_workload": 0,
                "disk_available_least": 0,
                "free_disk_gb": 1028,
                "free_ram_mb": 7680,
                "local_gb": 1028,
                "local_gb_used": 0,
                "memory_mb": 8192,
                "memory_mb_used": 512,
                "running_vms": 0,
                "vcpus": 1,
                "vcpus_used": 0}}
        self.check_service_client_function(
            self.client.show_hypervisor_statistics,
            'tempest.lib.common.rest_client.RestClient.get',
            expected, bytes_body)

    def test_show_hypervisor_uptime_str_body(self):
        self._test_show_hypervisor_uptime(bytes_body=False)

    def test_show_hypervisor_uptime_byte_body(self):
        self._test_show_hypervisor_uptime(bytes_body=True)

    def _test_show_hypervisor_uptime(self, bytes_body=False):
        expected = {
            "hypervisor": {
                "hypervisor_hostname": "fake-mini",
                "id": 1,
                "uptime": (" 08:32:11 up 93 days, 18:25, 12 users, "
                           " load average: 0.20, 0.12, 0.14")
            }}
        self.check_service_client_function(
            self.client.show_hypervisor_uptime,
            'tempest.lib.common.rest_client.RestClient.get',
            expected, bytes_body,
            hypervisor_id=self.hypervisor_id)

    def test_search_hypervisor_str_body(self):
        self._test_search_hypervisor(bytes_body=False)

    def test_search_hypervisor_byte_body(self):
        self._test_search_hypervisor(bytes_body=True)

    def _test_search_hypervisor(self, bytes_body=False):
        expected = {"hypervisors": [{
            "id": 2,
            "hypervisor_hostname": "hyper.hostname.com"}]}
        self.check_service_client_function(
            self.client.search_hypervisor,
            'tempest.lib.common.rest_client.RestClient.get',
            expected, bytes_body,
            hypervisor_name=self.hypervisor_name)



# Copyright 2019 NEC Corporation.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common import api_microversion_fixture
from tempest.lib.services.compute import base_compute_client
from tempest.lib.services.placement import base_placement_client
from tempest.lib.services.volume import base_client
from tempest.tests import base


class TestAPIMicroversionFixture(base.TestCase):
    def setUp(self):
        super(TestAPIMicroversionFixture, self).setUp()
        # Verify that all the microversion are reset back to None
        # by Fixture.
        self.assertIsNone(base_compute_client.COMPUTE_MICROVERSION)
        self.assertIsNone(base_client.VOLUME_MICROVERSION)
        self.assertIsNone(base_placement_client.PLACEMENT_MICROVERSION)

    def test_compute_microversion(self):
        self.useFixture(api_microversion_fixture.APIMicroversionFixture(
            compute_microversion='2.10'))
        self.assertEqual('2.10', base_compute_client.COMPUTE_MICROVERSION)
        self.assertIsNone(base_client.VOLUME_MICROVERSION)
        self.assertIsNone(base_placement_client.PLACEMENT_MICROVERSION)

    def test_volume_microversion(self):
        self.useFixture(api_microversion_fixture.APIMicroversionFixture(
            volume_microversion='3.10'))
        self.assertIsNone(base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.10', base_client.VOLUME_MICROVERSION)
        self.assertIsNone(base_placement_client.PLACEMENT_MICROVERSION)

    def test_placement_microversion(self):
        self.useFixture(api_microversion_fixture.APIMicroversionFixture(
            placement_microversion='1.10'))
        self.assertIsNone(base_compute_client.COMPUTE_MICROVERSION)
        self.assertIsNone(base_client.VOLUME_MICROVERSION)
        self.assertEqual('1.10', base_placement_client.PLACEMENT_MICROVERSION)

    def test_multiple_service_microversion(self):
        self.useFixture(api_microversion_fixture.APIMicroversionFixture(
            compute_microversion='2.10', volume_microversion='3.10',
            placement_microversion='1.10'))
        self.assertEqual('2.10', base_compute_client.COMPUTE_MICROVERSION)
        self.assertEqual('3.10', base_client.VOLUME_MICROVERSION)
        self.assertEqual('1.10', base_placement_client.PLACEMENT_MICROVERSION)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc


class ServerMetadataNegativeTestJSON(base.BaseV2ComputeTest):
    """Negative tests of server metadata"""

    create_default_network = True

    @classmethod
    def setup_clients(cls):
        super(ServerMetadataNegativeTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServerMetadataNegativeTestJSON, cls).resource_setup()
        cls.tenant_id = cls.client.tenant_id
        cls.server = cls.create_test_server(metadata={}, wait_until='ACTIVE')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('fe114a8f-3a57-4eff-9ee2-4e14628df049')
    def test_server_create_metadata_key_too_long(self):
        """Test creating server with too long metadata key should fail"""
        # Attempt to start a server with a meta-data key that is > 255
        # characters

        # Try create a server with the metadata for a few values
        for sz in [256, 257, 511, 1023]:
            key = "k" * sz
            meta = {key: 'data1'}
            self.assertRaises((lib_exc.BadRequest, lib_exc.OverLimit),
                              self.create_test_server,
                              metadata=meta)

        # no teardown - all creates should fail

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('92431555-4d8b-467c-b95b-b17daa5e57ff')
    def test_create_server_metadata_blank_key(self):
        """Test creating server with blank metadata key should fail"""
        meta = {'': 'data1'}
        self.assertRaises(lib_exc.BadRequest,
                          self.create_test_server,
                          metadata=meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('4d9cd7a3-2010-4b41-b8fe-3bbf0b169466')
    def test_server_metadata_non_existent_server(self):
        """Test getting metadata item for a non existent server should fail"""
        # GET on a non-existent server should not succeed
        non_existent_server_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.client.show_server_metadata_item,
                          non_existent_server_id,
                          'test2')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('f408e78e-3066-4097-9299-3b0182da812e')
    def test_list_server_metadata_non_existent_server(self):
        """Test listing metadata for a non existent server should fail"""
        non_existent_server_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.client.list_server_metadata,
                          non_existent_server_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0025fbd6-a4ba-4cde-b8c2-96805dcfdabc')
    def test_set_metadata_invalid_key(self):
        """Test setting server metadata item with wrong key in body

        Raise BadRequest if key in uri does not match the key passed in body.
        """
        meta = {'testkey': 'testvalue'}
        self.assertRaises(lib_exc.BadRequest,
                          self.client.set_server_metadata_item,
                          self.server['id'], 'key', meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('0df38c2a-3d4e-4db5-98d8-d4d9fa843a12')
    def test_set_metadata_non_existent_server(self):
        """Test setting metadata for a non existent server should fail"""
        non_existent_server_id = data_utils.rand_uuid()
        meta = {'meta1': 'data1'}
        self.assertRaises(lib_exc.NotFound,
                          self.client.set_server_metadata,
                          non_existent_server_id,
                          meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('904b13dc-0ef2-4e4c-91cd-3b4a0f2f49d8')
    def test_update_metadata_non_existent_server(self):
        """Test updating metadata for a non existent server should fail"""
        non_existent_server_id = data_utils.rand_uuid()
        meta = {'key1': 'value1', 'key2': 'value2'}
        self.assertRaises(lib_exc.NotFound,
                          self.client.update_server_metadata,
                          non_existent_server_id,
                          meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a452f38c-05c2-4b47-bd44-a4f0bf5a5e48')
    def test_update_metadata_with_blank_key(self):
        """Test updating server metadata to blank key should fail"""
        meta = {'': 'data1'}
        self.assertRaises(lib_exc.BadRequest,
                          self.client.update_server_metadata,
                          self.server['id'], meta=meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6bbd88e1-f8b3-424d-ba10-ae21c45ada8d')
    def test_delete_metadata_non_existent_server(self):
        """Test deleting metadata item from a non existent server

        Should not be able to delete metadata item from a non-existent server.
        """
        non_existent_server_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.client.delete_server_metadata_item,
                          non_existent_server_id,
                          'd')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d8c0a210-a5c3-4664-be04-69d96746b547')
    def test_metadata_items_limit(self):
        """Test set/update server metadata over limit should fail

        A 403 Forbidden or 413 Overlimit (old behaviour) exception
        will be raised while exceeding metadata items limit for project.
        """
        quota_set = self.quotas_client.show_quota_set(
            self.tenant_id)['quota_set']
        quota_metadata = quota_set['metadata_items']
        if quota_metadata == -1:
            raise self.skipException("No limit for metadata_items")

        req_metadata = {}
        for num in range(1, quota_metadata + 2):
            req_metadata['key' + str(num)] = 'val' + str(num)
        self.assertRaises((lib_exc.OverLimit, lib_exc.Forbidden),
                          self.client.set_server_metadata,
                          self.server['id'], req_metadata)

        # A 403 Forbidden or 413 Overlimit (old behaviour) exception
        # will be raised while exceeding metadata items limit for
        # tenant.
        self.assertRaises((lib_exc.Forbidden, lib_exc.OverLimit),
                          self.client.update_server_metadata,
                          self.server['id'], req_metadata)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('96100343-7fa9-40d8-80fa-d29ef588ce1c')
    def test_set_server_metadata_blank_key(self):
        """Test setting server metadata with blank key should fail"""
        meta = {'': 'data1'}
        self.assertRaises(lib_exc.BadRequest,
                          self.client.set_server_metadata,
                          self.server['id'], meta=meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('64a91aee-9723-4863-be44-4c9d9f1e7d0e')
    def test_set_server_metadata_missing_metadata(self):
        """Test setting server metadata without metadata field should fail"""
        meta = {'meta1': 'data1'}
        self.assertRaises(lib_exc.BadRequest,
                          self.client.set_server_metadata,
                          self.server['id'], meta=meta, no_metadata_field=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class BaseNetworkClient(rest_client.RestClient):

    """Base class for Tempest REST clients for Neutron.

    Child classes use v2 of the Neutron API, since the V1 API has been
    removed from the code base.
    """

    version = '2.0'
    uri_prefix = "v2.0"

    def list_resources(self, uri, **filters):
        req_uri = self.uri_prefix + uri
        if filters:
            req_uri += '?' + urllib.urlencode(filters, doseq=1)
        resp, body = self.get(req_uri)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_resource(self, uri):
        req_uri = self.uri_prefix + uri
        resp, body = self.delete(req_uri)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def show_resource(self, uri, **fields):
        # fields is a dict which key is 'fields' and value is a
        # list of field's name. An example:
        # {'fields': ['id', 'name']}
        req_uri = self.uri_prefix + uri
        if fields:
            req_uri += '?' + urllib.urlencode(fields, doseq=1)
        resp, body = self.get(req_uri)
        body = json.loads(body)
        self.expected_success(200, resp.status)
        return rest_client.ResponseBody(resp, body)

    def create_resource(self, uri, post_data, expect_empty_body=False,
                        expect_response_code=201):
        req_uri = self.uri_prefix + uri
        req_post_data = json.dumps(post_data)
        resp, body = self.post(req_uri, req_post_data)
        # NOTE: RFC allows both a valid non-empty body and an empty body for
        # response of POST API. If a body is expected not empty, we decode the
        # body. Otherwise we returns the body as it is.
        if not expect_empty_body:
            body = json.loads(body)
        else:
            body = None
        self.expected_success(expect_response_code, resp.status)
        return rest_client.ResponseBody(resp, body)

    def update_resource(self, uri, post_data, expect_empty_body=False,
                        expect_response_code=200):
        req_uri = self.uri_prefix + uri
        req_post_data = json.dumps(post_data)
        resp, body = self.put(req_uri, req_post_data)
        # NOTE: RFC allows both a valid non-empty body and an empty body for
        # response of PUT API. If a body is expected not empty, we decode the
        # body. Otherwise we returns the body as it is.
        if not expect_empty_body:
            body = json.loads(body)
        else:
            body = None
        self.expected_success(expect_response_code, resp.status)
        return rest_client.ResponseBody(resp, body)



# Copyright 2024 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import testscenarios
import yaml

from tempest.api.compute import base as compute_base
from tempest.api.image import base
from tempest.common import waiters
from tempest import config
from tempest import exceptions
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


def load_tests(loader, suite, pattern):
    """Generate scenarios from the image manifest."""
    if CONF.image.images_manifest_file is None:
        return suite
    ImagesFormatTest.scenarios = []
    with open(CONF.image.images_manifest_file) as f:
        ImagesFormatTest._manifest = yaml.load(f, Loader=yaml.SafeLoader)
        for imgdef in ImagesFormatTest._manifest['images']:
            ImagesFormatTest.scenarios.append((imgdef['name'],
                                               {'imgdef': imgdef}))
    result = loader.suiteClass()
    result.addTests(testscenarios.generate_scenarios(suite))
    return result


class ImagesFormatTest(base.BaseV2ImageTest,
                       compute_base.BaseV2ComputeTest):
    def setUp(self):
        super().setUp()
        if CONF.image.images_manifest_file is None:
            self.skipTest('Image format testing is not configured')
        self._image_base = os.path.dirname(os.path.abspath(
            CONF.image.images_manifest_file))

        self.images = []

    def tearDown(self):
        for img in self.images:
            try:
                self.client.delete_image(img['id'])
            except lib_exc.NotFound:
                pass
        return super().tearDown()

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        cls.available_import_methods = cls.client.info_import()[
            'import-methods']['value']

    def _test_image(self, image_def, override_format=None, asimport=False):
        image_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name=image_def['name'])
        image = self.client.create_image(
            name=image_name,
            container_format='bare',
            disk_format=override_format or image_def['format'])
        self.images.append(image)
        image_fn = os.path.join(self._image_base, image_def['filename'])
        with open(image_fn, 'rb') as f:
            if asimport:
                self.client.stage_image_file(image['id'], f)
                self.client.image_import(image['id'], method='glance-direct')
            else:
                self.client.store_image_file(image['id'], f)
        return image

    @decorators.idempotent_id('a245fcbe-63ce-4dc1-a1d0-c16d76d9e6df')
    def test_accept_usable_formats(self):
        if self.imgdef['usable']:
            try:
                self._test_image(self.imgdef)
            except lib_exc.BadRequest:
                format = self.imgdef['format']
                if format == 'gpt' and format not in CONF.image.disk_formats:
                    # If we don't have gpt defined, we don't expect this to
                    # work because glance has not been updated for GPT
                    # FIXME(danms): Remove this once glance support for GPT is
                    # landed on master
                    self.skipTest('GPT not configured and glance is too '
                                  'old to support it')
                raise
        else:
            self.skipTest(
                'Glance does not currently reject unusable images on upload')

    @decorators.idempotent_id('7c7c2f16-2e97-4dce-8cb4-bc10be031c85')
    def test_accept_reject_formats_import(self):
        """Make sure glance rejects invalid images during conversion."""
        if 'glance-direct' not in self.available_import_methods:
            self.skipTest('Import via glance-direct is not available')
        if not CONF.image_feature_enabled.image_conversion:
            self.skipTest('Import image_conversion not enabled')

        # VMDK with footer was not supported by earlier service versions,
        # so we need to tolerate it passing and failing (skip for the latter).
        # See this for more info:
        # https://bugs.launchpad.net/glance/+bug/2073262
        is_broken = 'footer' in self.imgdef['name']

        if (self.imgdef['format'] in CONF.image.disk_formats and
                self.imgdef['usable']):
            # Usable images should end up in active state
            image = self._test_image(self.imgdef, asimport=True)
            try:
                waiters.wait_for_image_status(self.client, image['id'],
                                              'active')
            except lib_exc.TimeoutException:
                if is_broken:
                    self.skipTest(
                        'Older glance did not support vmdk-with-footer')
                else:
                    raise
        else:
            # FIXME(danms): Make this better, but gpt will fail before
            # the import even starts until glance has it in its API
            # schema as a valid value. Other formats expected to fail
            # do so during import and return to queued state.
            try:
                image = self._test_image(self.imgdef, asimport=True)
                waiters.wait_for_image_status(self.client, image['id'],
                                              ['queued', 'active'])
                self.client.delete_image(image['id'])
            except lib_exc.BadRequest:
                format = self.imgdef['format']
                if format == 'gpt' and format not in CONF.image.disk_formats:
                    # If we don't have gpt defined, we don't expect this to
                    # work because glance has not been updated for GPT
                    # FIXME(danms): Remove this once glance support for GPT is
                    # landed on master
                    self.skipTest('GPT not configured and glance is too '
                                  'old to support it')
                elif format in CONF.image.disk_formats:
                    # This is in our config so it's supposed to work, fail
                    raise

        if self.imgdef['format'] == 'iso':
            # NOTE(danms): Glance has a special case to not convert ISO images
            # because they are special and must remain as ISOs in order to be
            # properly used for CD-based rescue and boot.
            self.assertEqual('iso', image['disk_format'])

    def _create_server_with_image_def(self, image_def, **overrides):
        image_def = dict(image_def, **overrides)
        image = self._test_image(image_def)
        server = self.create_test_server(name='server-%s' % image['name'],
                                         image_id=image['id'],
                                         wait_until='ACTIVE')
        return server

    @decorators.idempotent_id('f77394bc-81f4-4d54-9f5b-e48f3d6b5376')
    def test_compute_rejects_invalid(self):
        """Make sure compute rejects invalid/insecure images."""
        if self.imgdef['format'] not in CONF.image.disk_formats:
            # if this format is not allowed by glance, we can not create
            # a properly-formatted image for it, so skip it.
            self.skipTest(
                'Format %s not allowed by config' % self.imgdef['format'])
        if CONF.image_feature_enabled.image_format_enforcement:
            # If glance rejects bad images during upload, we cannot get them
            # registered so that we can test nova.
            self.skipTest(
                'Unable to test compute image formats if glance does not '
                'allow them to be uploaded')

        # VMDK with footer was not supported by earlier service versions,
        # so we need to tolerate it passing and failing (skip for the latter).
        # See this for more info:
        # https://bugs.launchpad.net/glance/+bug/2073262
        is_broken = 'footer' in self.imgdef['name']

        if self.imgdef['usable']:
            try:
                server = self._create_server_with_image_def(self.imgdef)
            except exceptions.BuildErrorException:
                if is_broken:
                    self.skip('Tolerating failed build with known-broken '
                              'image format')
                else:
                    raise
            self.delete_server(server['id'])
        else:
            self.assertRaises(exceptions.BuildErrorException,
                              self._create_server_with_image_def,
                              self.imgdef)

    @decorators.idempotent_id('ffe21610-e801-4992-9b81-e2d646e2e2e9')
    def test_compute_rejects_format_mismatch(self):
        """Make sure compute rejects any image with a format mismatch."""
        if CONF.image_feature_enabled.image_format_enforcement:
            # If glance rejects bad images during upload, we cannot get them
            # registered so that we can test nova.
            self.skipTest(
                'Unable to test compute image formats if glance does not '
                'allow them to be uploaded')
        # Lying about the disk_format should always fail
        override_fmt = (
            self.imgdef['format'] in ('raw', 'gpt') and 'qcow2' or 'raw')
        self.assertRaises(exceptions.BuildErrorException,
                          self._create_server_with_image_def,
                          self.imgdef,
                          format=override_fmt)



# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import functools
from urllib import parse as urllib

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client
from tempest.lib import exceptions as lib_exc

CHUNKSIZE = 1024 * 64  # 64kB


class ImagesClient(rest_client.RestClient):
    api_version = "v2"

    def update_image(self, image_id, patch):
        """Update an image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#update-image
        """
        data = json.dumps(patch)
        headers = {"Content-Type": "application/openstack-images-v2.0"
                                   "-json-patch"}
        resp, body = self.patch('images/%s' % image_id, data, headers)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def create_image(self, **kwargs):
        """Create an image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#create-image
        """
        data = json.dumps(kwargs)
        resp, body = self.post('images', data)
        self.expected_success(201, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def deactivate_image(self, image_id):
        """Deactivate image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#deactivate-image
        """
        url = 'images/%s/actions/deactivate' % image_id
        resp, body = self.post(url, None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def reactivate_image(self, image_id):
        """Reactivate image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#reactivate-image
        """
        url = 'images/%s/actions/reactivate' % image_id
        resp, body = self.post(url, None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_image(self, image_id):
        """Delete image.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#delete-image
         """
        url = 'images/%s' % image_id
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def list_images(self, params=None):
        """List images.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#list-images
        """
        url = 'images'

        if params:
            url += '?%s' % urllib.urlencode(params)

        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_image(self, image_id):
        """Show image details.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#show-image
        """
        url = 'images/%s' % image_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def show_image_tasks(self, image_id):
        """Show image tasks."""
        url = 'images/%s/tasks' % image_id
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def is_resource_deleted(self, id):
        try:
            self.show_image(id)
        except lib_exc.NotFound:
            return True
        return False

    def is_resource_active(self, id):
        try:
            image = self.show_image(id)
            if image['status'] != 'active':
                return False
        except lib_exc.NotFound:
            return False
        return True

    @property
    def resource_type(self):
        """Returns the primary type of resource this client works with."""
        return 'image'

    def store_image_file(self, image_id, data):
        """Upload binary image data.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#upload-binary-image-data
        """
        url = 'images/%s/file' % image_id

        # We are going to do chunked transfer, so split the input data
        # info fixed-sized chunks.
        headers = {'Content-Type': 'application/octet-stream'}
        data = iter(functools.partial(data.read, CHUNKSIZE), b'')

        resp, body = self.request('PUT', url, headers=headers,
                                  body=data, chunked=True)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def stage_image_file(self, image_id, data):
        """Upload binary image data to staging area.

        For a full list of available parameters, please refer to the official
        API reference (stage API:
        https://docs.openstack.org/api-ref/image/v2/#interoperable-image-import
        """
        url = 'images/%s/stage' % image_id

        # We are going to do chunked transfer, so split the input data
        # info fixed-sized chunks.
        headers = {'Content-Type': 'application/octet-stream'}
        data = iter(functools.partial(data.read, CHUNKSIZE), b'')

        resp, body = self.request('PUT', url, headers=headers,
                                  body=data, chunked=True)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def info_import(self):
        """Return information about server-supported import methods."""
        url = 'info/import'
        resp, body = self.get(url)

        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def info_stores(self):
        """Return information about server-supported stores."""
        url = 'info/stores'
        resp, body = self.get(url)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def image_import(self, image_id, method='glance-direct',
                     all_stores_must_succeed=None, all_stores=True,
                     stores=None, import_params=None):
        """Import data from staging area to glance store.

        For a full list of available parameters, please refer to the official
        API reference (stage API:
        https://docs.openstack.org/api-ref/image/v2/#interoperable-image-import

        :param method: The import method (i.e. glance-direct) to use
        :param all_stores_must_succeed: Boolean indicating if all store imports
                                        must succeed for the import to be
                                        considered successful. Must be None if
                                        server does not support multistore.
        :param all_stores: Boolean indicating if image should be imported to
                           all available stores (incompatible with stores)
        :param stores: A list of destination store names for the import. Must
                       be None if server does not support multistore.
        :param import_params: A dict of import method parameters
        """
        url = 'images/%s/import' % image_id
        if import_params is None:
            import_params = {}
        data = {
            "method": {
                "name": method
            },
        }
        if stores is not None:
            data["stores"] = stores
        else:
            data["all_stores"] = all_stores

        if all_stores_must_succeed is not None:
            data['all_stores_must_succeed'] = all_stores_must_succeed
        if import_params:
            data['method'].update(import_params)
        data = json.dumps(data)
        headers = {'Content-Type': 'application/json'}
        resp, _ = self.post(url, data, headers=headers)

        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp)

    def show_image_file(self, image_id, chunked=False):
        """Download binary image data.

        :param bool chunked: If True, do not read the body and return only
                             the raw urllib3 response object for processing.
                             NB: If you pass True here, you **MUST** call
                             release_conn() on the response object before
                             finishing!

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#download-binary-image-data
        """
        url = 'images/%s/file' % image_id
        resp, body = self.get(url, chunked=chunked)
        self.expected_success([200, 204, 206], resp.status)
        if chunked:
            return resp
        else:
            return rest_client.ResponseBodyData(resp, body)

    def add_image_tag(self, image_id, tag):
        """Add an image tag.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#add-image-tag
        """
        url = 'images/%s/tags/%s' % (image_id, tag)
        resp, body = self.put(url, body=None)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp, body)

    def delete_image_tag(self, image_id, tag):
        """Delete an image tag.

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/#delete-image-tag
        """
        url = 'images/%s/tags/%s' % (image_id, tag)
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def delete_image_from_store(self, image_id, store_name):
        """Delete image from store

        For a full list of available parameters,
        please refer to the official API reference:
        https://docs.openstack.org/api-ref/image/v2/#delete-image-from-store
        """
        url = 'stores/%s/%s' % (store_name, image_id)
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)



# Copyright 2014 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.tests.lib import fake_http

FAKE_AUTH_URL = 'http://fake_uri.com/auth'

TOKEN = "fake_token"
ALT_TOKEN = "alt_fake_token"

# Fake Identity v2 constants
COMPUTE_ENDPOINTS_V2 = {
    "endpoints": [
        {
            "adminURL": "http://fake_url/v2/first_endpoint/admin",
            "region": "NoMatchRegion",
            "internalURL": "http://fake_url/v2/first_endpoint/internal",
            "publicURL": "http://fake_url/v2/first_endpoint/public"
        },
        {
            "adminURL": "http://fake_url/v2/second_endpoint/admin",
            "region": "FakeRegion",
            "internalURL": "http://fake_url/v2/second_endpoint/internal",
            "publicURL": "http://fake_url/v2/second_endpoint/public"
        },
    ],
    "type": "compute",
    "name": "nova"
}

CATALOG_V2 = [COMPUTE_ENDPOINTS_V2, ]

ALT_IDENTITY_V2_RESPONSE = {
    "access": {
        "token": {
            "expires": "2020-01-01T00:00:10Z",
            "id": ALT_TOKEN,
            "tenant": {
                "id": "fake_alt_tenant_id"
            },
        },
        "user": {
            "id": "fake_alt_user_id",
            "password_expires_at": None,
        },
        "serviceCatalog": CATALOG_V2,
    },
}

IDENTITY_V2_RESPONSE = {
    "access": {
        "token": {
            "expires": "2020-01-01T00:00:10Z",
            "id": TOKEN,
            "tenant": {
                "id": "fake_tenant_id"
            },
        },
        "user": {
            "id": "fake_user_id",
            "password_expires_at": None,
        },
        "serviceCatalog": CATALOG_V2,
    },
}

# Fake Identity V3 constants
COMPUTE_ENDPOINTS_V3 = {
    "endpoints": [
        {
            "id": "first_compute_fake_service",
            "interface": "public",
            "region": "NoMatchRegion",
            "region_id": "NoMatchRegion",
            "url": "http://fake_url/v3/first_endpoint/api"
        },
        {
            "id": "second_fake_service",
            "interface": "public",
            "region": "FakeRegion",
            "region_id": "FakeRegion",
            "url": "http://fake_url/v3/second_endpoint/api"
        },
        {
            "id": "third_fake_service",
            "interface": "admin",
            "region": "MiddleEarthRegion",
            "region_id": "MiddleEarthRegion",
            "url": "http://fake_url/v3/third_endpoint/api"
        }

    ],
    "type": "compute",
    "id": "fake_compute_endpoint",
    "name": "nova"
}

CATALOG_V3 = [COMPUTE_ENDPOINTS_V3, ]

IDENTITY_V3_RESPONSE = {
    "token": {
        "audit_ids": ["ny5LA5YXToa_mAVO8Hnupw", "9NPTvsRDSkmsW61abP978Q"],
        "methods": [
            "token",
            "password"
        ],
        "expires_at": "2020-01-01T00:00:10.000123Z",
        "project": {
            "domain": {
                "id": "fake_domain_id",
                "name": "fake"
            },
            "id": "project_id",
            "name": "project_name"
        },
        "user": {
            "domain": {
                "id": "fake_domain_id",
                "name": "domain_name"
            },
            "id": "fake_user_id",
            "name": "username",
            "password_expires_at": None,
        },
        "issued_at": "2013-05-29T16:55:21.468960Z",
        "catalog": CATALOG_V3
    }
}

IDENTITY_V3_RESPONSE_DOMAIN_SCOPE = {
    "token": {
        "audit_ids": ["ny5LA5YXToa_mAVO8Hnupw", "9NPTvsRDSkmsW61abP978Q"],
        "methods": [
            "token",
            "password"
        ],
        "expires_at": "2020-01-01T00:00:10.000123Z",
        "domain": {
            "id": "fake_domain_id",
            "name": "domain_name"
        },
        "user": {
            "domain": {
                "id": "fake_domain_id",
                "name": "domain_name"
            },
            "id": "fake_user_id",
            "name": "username",
            "password_expires_at": None,
        },
        "issued_at": "2013-05-29T16:55:21.468960Z",
        "catalog": CATALOG_V3
    }
}

IDENTITY_V3_RESPONSE_NO_SCOPE = {
    "token": {
        "audit_ids": ["ny5LA5YXToa_mAVO8Hnupw", "9NPTvsRDSkmsW61abP978Q"],
        "methods": [
            "token",
            "password"
        ],
        "expires_at": "2020-01-01T00:00:10.000123Z",
        "user": {
            "domain": {
                "id": "fake_domain_id",
                "name": "domain_name"
            },
            "id": "fake_user_id",
            "name": "username",
            "password_expires_at": None,
        },
        "issued_at": "2013-05-29T16:55:21.468960Z",
    }
}

ALT_IDENTITY_V3 = IDENTITY_V3_RESPONSE


def _fake_v3_response(self, uri, method="GET", body=None, headers=None,
                      redirections=5, connection_type=None, log_req_body=None):
    fake_headers = {
        "x-subject-token": TOKEN
    }
    return (fake_http.fake_http_response(fake_headers, status=201),
            json.dumps(IDENTITY_V3_RESPONSE))


def _fake_v3_response_domain_scope(self, uri, method="GET", body=None,
                                   headers=None, redirections=5,
                                   connection_type=None, log_req_body=None):
    fake_headers = {
        "status": "201",
        "x-subject-token": TOKEN
    }
    return (fake_http.fake_http_response(fake_headers, status=201),
            json.dumps(IDENTITY_V3_RESPONSE_DOMAIN_SCOPE))


def _fake_v3_response_no_scope(self, uri, method="GET", body=None,
                               headers=None, redirections=5,
                               connection_type=None, log_req_body=None):
    fake_headers = {
        "status": "201",
        "x-subject-token": TOKEN
    }
    return (fake_http.fake_http_response(fake_headers, status=201),
            json.dumps(IDENTITY_V3_RESPONSE_NO_SCOPE))


def _fake_v2_response(self, uri, method="GET", body=None, headers=None,
                      redirections=5, connection_type=None, log_req_body=None):
    return (fake_http.fake_http_response({}, status=200),
            json.dumps(IDENTITY_V2_RESPONSE))


def _fake_auth_failure_response():
    # the response body isn't really used in this case, but lets send it anyway
    # to have a safe check in some future change on the rest client.
    body = {
        "unauthorized": {
            "message": "Unauthorized",
            "code": "401"
        }
    }
    return fake_http.fake_http_response({}, status=401), json.dumps(body)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

get_limit = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'limits': {
                'type': 'object',
                'properties': {
                    'absolute': {
                        'type': 'object',
                        'properties': {
                            'maxTotalRAMSize': {'type': 'integer'},
                            'totalCoresUsed': {'type': 'integer'},
                            'maxTotalInstances': {'type': 'integer'},
                            'maxTotalFloatingIps': {'type': 'integer'},
                            'totalSecurityGroupsUsed': {'type': 'integer'},
                            'maxTotalCores': {'type': 'integer'},
                            'totalFloatingIpsUsed': {'type': 'integer'},
                            'maxSecurityGroups': {'type': 'integer'},
                            'maxServerMeta': {'type': 'integer'},
                            'maxPersonality': {'type': 'integer'},
                            'maxImageMeta': {'type': 'integer'},
                            'maxPersonalitySize': {'type': 'integer'},
                            'maxSecurityGroupRules': {'type': 'integer'},
                            'maxTotalKeypairs': {'type': 'integer'},
                            'totalRAMUsed': {'type': 'integer'},
                            'totalInstancesUsed': {'type': 'integer'},
                            'maxServerGroupMembers': {'type': 'integer'},
                            'maxServerGroups': {'type': 'integer'},
                            'totalServerGroupsUsed': {'type': 'integer'}
                        },
                        'additionalProperties': False,
                        # NOTE(gmann): maxServerGroupMembers,  maxServerGroups
                        # and totalServerGroupsUsed are API extension,
                        # and some environments return a response without these
                        # attributes.So they are not 'required'.
                        'required': ['maxImageMeta',
                                     'maxPersonality',
                                     'maxPersonalitySize',
                                     'maxSecurityGroupRules',
                                     'maxSecurityGroups',
                                     'maxServerMeta',
                                     'maxTotalCores',
                                     'maxTotalFloatingIps',
                                     'maxTotalInstances',
                                     'maxTotalKeypairs',
                                     'maxTotalRAMSize',
                                     'totalCoresUsed',
                                     'totalFloatingIpsUsed',
                                     'totalInstancesUsed',
                                     'totalRAMUsed',
                                     'totalSecurityGroupsUsed']
                    },
                    'rate': {
                        'type': 'array',
                        'items': {
                            'type': 'object',
                            'properties': {
                                'limit': {
                                    'type': 'array',
                                    'items': {
                                        'type': 'object',
                                        'properties': {
                                            'next-available':
                                                {'type': 'string'},
                                            'remaining':
                                                {'type': 'integer'},
                                            'unit':
                                                {'type': 'string'},
                                            'value':
                                                {'type': 'integer'},
                                            'verb':
                                                {'type': 'string'}
                                        },
                                        'additionalProperties': False,
                                    }
                                },
                                'regex': {'type': 'string'},
                                'uri': {'type': 'string'}
                            },
                            'additionalProperties': False,
                        }
                    }
                },
                'additionalProperties': False,
                'required': ['absolute', 'rate']
            }
        },
        'additionalProperties': False,
        'required': ['limits']
    }
}
